// delay.c
//
// TV Remote controller project for Motorola68HC908QY4
// Based on SDCC M08 Port Compiler & Library
// By Sarosi, 2004.03.07, Free
//

#include "delay.h"


//-----------------------------------------------
//Clock trimmed to 3 Mega cyces/sec, so 3 cycles => 1us 
//-----------------------------------------------
// _TrimByteContainer - container proc only, never called!
//-----------------------------------------------
static void _TrimByteContainer(void){
_asm
   .area _ABSOLUTE(ABS)
   .org 0xffc0
   .db 0x82
   .area _CODE
_endasm;
}


//-----------------------------------------------
// Delay1us - in range 5...255 [us]
//-----------------------------------------------
//Parameter us in range 0..4 causes 5us delay
void Delay1us(unsigned char us){
   us+0;
   _asm                 // 7c  (load+call)
        sub #5          // 2c  
        bls delay1us_e  // 2c  jump if we are over 5us=15tick (load/call/sub/ret)
        
   delay1us_c:
        dbnza delay1us_c // 3c

   delay1us_e:
        ;rts            // 4c
_endasm;
}


//-----------------------------------------------
// Delay10us - in range 1...255  [10us]
//-----------------------------------------------
//First 10us is eaten up by the call and return, so must be adjusted.
//Parameter 0 causes 5us delay
void Delay10us(unsigned char tus){
   tus+0;
   _asm                    // 7c  (load+call)
         sub #1            // 2c
         bls delay10us_e	// 2c  jump if we are over 5us=15tick (load/call/sub/ret)

         tax               // 1c

         lda #5            //
         bsr _delay1us     // 14c = 9 us -1c

   delay10us_c:
         lda #9            // 27c = 9 us
         jsr _delay1us     //

         dbnzx delay10us_c // 3c = 1 us

   delay10us_e:
         ;rts              // 4c
_endasm;
}


/*

( Trimming:
  90 sec 4x5000 ms

 tick = 18+N*(29 + 13*K)= 18 + 20000*(29 + 13*1024) = 266 820 018

 266 820 018 tick => 90 sec
  1 tick => 90*1 000 000 / 266 820 018 usec/tick = 0.3373 usec/tick
  1 sec => 266 820 018 / 90 tick/sec = 2 964 667 tick/sec

  We trim to 3Mhz, so  trim value = 127*(3/2.9646)  -> 129 ->0x82

tick = 18+N*(29 + 13*K)= 18 + 20000*(29 + 13*K) = 20*3000000 = 60 000 000
 (60 000 000 - 18)/20000 = 13*k +29
((60 000 000 - 18)/20000 -29)/13= k = 229 (1024 helyett)

*/

//-----------------------------------------------
// DelayMs - in range 0..65535 ms
//-----------------------------------------------
void DelayMs(unsigned int ms){
   ms+0;
   _asm                    // 13(call)
             pshx          // 2
             tax           // 1
             pulh          // 2

   delay:
             cphx #0       // 3
             beq  delay_e  // 2
             aix  #-1      // 2

             pshx          // 2
             pshh          // 2
             ldhx #229     // 3

   delay_tc: 
             sta 0xffff    // 4

             cphx #0       // 3
             beq  delay_te // 2
             aix  #-1      // 2
             bra delay_tc  // 2

   delay_te: 
             pulh          // 2
             pulx          // 2
             bra  delay    // 2

   delay_e:  ;rts          // 4
_endasm;
}


// End of delay.c

