;
;  ======================================================
;  =                  DEB08.A08                         =
;  ======================================================
;  =              Debugger for HC08                     =
;  ======================================================
;   Writte by:           Jozsef Sarosi
;   Started:             07-Jun-1999
;   Version:             1.04 (2003.09.12)
;
; -mus
; -b _ABS    = 0x0000  ; area for absolute addresses
; -b _DATA   = 0x0060  ; RAM page 0 segment
; -b _STACK  = 0x0200  ; RAM
; -b _XDATA  = 0x0A00  ; RAM
; -b _CODE   = 0x2000  ; FLASH segment


STANDALONE = 0 ;1=standalone; 0=under Bencsik Loader

FULL_SIZE = 0 ;1=full features 0=minimal size

SERIAL_FROM_MDEB08 = 0

;-------------------------------------------------------------;
; serial communication
;-------------------------------------------------------------;
SCI_channel = 1
BDM_channel = 2
;IO_channel  = SCI_channel
IO_channel  = BDM_channel

;-------------------------------------------------------------;
;                         Variables                           ;
;-------------------------------------------------------------;

         .area _DATA(PAGE0)

m_cim:   .ds  2       ; last used address in commands to remember
m_op1:   .ds  2       ; command line parameter 1, or overlapped
m_op2:   .ds  2       ; command line parameter 2, or overlapped
m_op3:   .ds  2       ; command line parameter 3, or overlapped
m_op4:   .ds  2       ; command line parameter 4, or overlapped
m_op5:   .ds  2       ; used by getline

m_pos:   .ds  2       ; used as command line index, or overlapped
.if FULL_SIZE
m_xpos   = 32	       ; maximal number of chars in command line
.else
m_xpos   = 16	       ; maximal number of chars in command line
.endif
m_line:  .ds  m_xpos  ; command line buffer

m_leng   =  m_line    ; byte list overlapped with line
m_list   =  m_line+1

m_inx    =  m_line    ; overlapped with line while it may be

m_beg  =  m_op1       ; overlaps to save RAM
m_end  =  m_op2
m_ind  =  m_op3
.if FULL_SIZE
m_db   =  m_op4
.endif

; actual state of user program
s_A:      .ds  1
s_CCR:    .ds  1
s_HX:     .ds  2
s_PC:     .ds  2
s_SP:     .ds  2

s_PCH     = s_PC
s_PCL     = s_PC+1
s_H       = s_HX
s_X       = s_HX+1
s_PCL     = s_PC+1


; processor constants and variables
SBFCR    =   0xfe03   ; SIM Break Flag Control Register
BFCE     =   0x80     ; Break Celar flag enable bit

BRKH     =   0xFE09   ; Break address MSB
BRKL     =   0xFE0A   ; Break address LSB
BRKSCR   =   0xFE0B   ; Break status and control register

PTA      =   0x00     ; port A
DDRA     =   0x04     ; port A data direction

PCTL     =   0x1c     ; Phase-Loked Loop (PLL) Control Reg.
PLLON    =   0x20     ; PLL on bit
LOCK     =   0x40     ; (r) Lock indicator bit

PLLIE    =   0x80     ; PLL interrupt enable bit
PLLF     =   0x40     ; (w) PLL interrupt flag
PLLON    =   0x20     ; PLL on bit
BCS      =   0x10     ; Base clock select bit

PBWC     =   0x1D     ;PLL Bandwidth Control Reg.
AUTO     =   0x80     ; Automatic bandwidth control bit

PPG      =   0x1E     ;PLL Programming Reg.

PTA      =   0x00     ; Port A Data Register
PTB      =   0x01     ; Port B Data Register
PTC      =   0x02     ; Port C Data Register
PTD      =   0x03     ; Port D Data Register
DDRA     =   0x04     ; Data Direction Register A
DDRB     =   0x05     ; Data Direction Register B
DDRC     =   0x06     ; Data Direction Register C
DDRD     =   0x07     ; Data Direction Register D

PTE      =   0x08     ; Port E Data Register
PTF      =   0x09     ; Port F Data Register
PTG      =   0x0a     ; Port G Data Register
PTH      =   0x0b     ; Port H Data Register
DDRE     =   0x0c     ; Data Direction Register E
DDRF     =   0x0d     ; Data Direction Register F
DDRG     =   0x0e     ; Data Direction Register G
DDRH     =   0x0f     ; Data Direction Register H

SPCR     = 0x10       ; Serial Peripheral Interface (SPI) Control Reg.

SPRIE     =   0x80   ; SPI Receiver interrupt enable
DMAS      =   0x40   ; DMA select
SPMSTR    =   0x20   ; SPI master
CPOL      =   0x10   ; Clock polarity
CPHA      =   0x08   ; Clock phase
SPWOM     =   0x04   ; SPI wired-OR mode
SPE       =   0x02   ; SPI enable
SPTIE     =   0x01   ; SPI transmit interrupt enable

SPSCR     =   0x11   ; SPI Status and Control Reg.

SPRF      =   0x80 ; SPI Receiver interrupt enable
ERRIE     =   0x40 ; Error interrupt enable
OVRF      =   0x20 ; Overflow flag
MODF      =   0x10 ; Mode fault
SPTE      =   0x08 ; SPI transmitter empty
MODFEN    =   0x04 ; Mode fault enable

SPSCR_BD_2    =  0x00  ; Baud rate master divisor 2
SPSCR_BD_8    =  0x01  ; Baud rate master divisor 8
SPSCR_BD_32   =  0x02  ; Baud rate master divisor 32
SPSCR_BD_128  =  0x03  ; Baud rate master divisor 128

  ; Baudrate = CGMOUT / 2 / SPSCR_BD

SPDR = 0x12      ; SPI Data Reg.

SCC1  = 0x13      ; Serial Communications Interface(SCI) Control Reg. 1

LOOPS = 0x80 ; Loop mode select bit
ENSCI = 0x40 ; Enable SCI bit
TXINV = 0x20 ; Transmit invertion bit
M     = 0x10 ; Mode (character length) bit (0=8, 1=9)
WAKE  = 0x08 ; Wake-up condition bit
ILTY  = 0x04 ; Idle line type bit
PEN   = 0x02 ; Parity enable bit
PTY   = 0x01 ; Parity bit (0=even, 1=odd)

SCC2  = 0x14    ; SCI Control Reg. 2

SCTIE  = 0x80 ; SCI transmit interrupt enable bit
TCIE   = 0x40 ; Transmission complet interrupt enable bit
SCRIE  = 0x20 ; SCI receive interrupt enable bit
ILIE   = 0x10 ; Idle line interrupt enable bit
TE     = 0x08 ; Transmitter enable bit
RE     = 0x04 ; Receiver enable bit
RWU    = 0x02 ; Receiver wake-up bit
SBK    = 0x01 ; Send break bit

SCC3 = 0x15  ; SCI Control Reg. 3

R8     = 0x80 ; Received bit 8
T8     = 0x40 ; Transmitted bit 8
DMARE  = 0x20 ; DMA receive enable bit
DMATE  = 0x10 ; DMA transfer enable bit
ORIE   = 0x08 ; Receiver overrun IT enable bit
NEIE   = 0x04 ; Receiver noise error IT enable bit
FEIE   = 0x02 ; Receiver framing error IT enable bit
PEIE   = 0x01 ; Receiver parity error IT enable bit

SCS1 = 0x16   ; SCI Status Reg. 1 (r)

SCTE = 0x80 ; SCI transmitter empty bit(mask)
SCTE_no = 7 ; SCI transmitter empty bit(no)

TC   = 0x40 ; Transmission complet bit(mask)
TC_no = 6   ; Transmission complet bit(no)

SCRF = 0x20 ; SCI receiver full bit(mask)
SCRF_no = 5 ; SCI receiver full bit(no)

IDLE = 0x10 ; Receiver idle bit
OR   = 0x08 ; Receiver overrun bit
NF   = 0x04 ; Receiver noise flag bit
FE   = 0x02 ; Receiver framing error bit
PE   = 0x01 ; Receiver parity error bit

SCS2 = 0x17 ; SCI Status Reg. 2 (r)

BKF  = 0x02 ; Break flag bit
RPF  = 0x01 ; Reception in progress flag bit


SCDR = 0x18 ; SCI Data Reg. (r/w)

SCBR = 0x19 ; SCI Baud Rate Reg.

SCBR_PD_1  = 0x00  ; Baud rate prescaling 1
SCBR_PD_3  = 0x10  ; Baud rate prescaling 3
SCBR_PD_4  = 0x20  ; Baud rate prescaling 4
SCBR_PD_13 = 0x30  ; Baud rate prescaling 13

SCBR_BD_1   = 0x00  ; Baud rate scaling 1
SCBR_BD_2   = 0x01  ; Baud rate scaling 2
SCBR_BD_4   = 0x02  ; Baud rate scaling 4
SCBR_BD_8   = 0x03  ; Baud rate scaling 8
SCBR_BD_16  = 0x04  ; Baud rate scaling 16
SCBR_BD_32  = 0x05  ; Baud rate scaling 32
SCBR_BD_64  = 0x06  ; Baud rate scaling 64
SCBR_BD_128 = 0x07  ; Baud rate scaling 128


ISCR = 0x1a      ; IRQ Status and Control Reg. (r/w)

IRQF1  = 0x08 ; IRQ1 flag (1=IRQ1 IT is pending)
ACK1   = 0x04 ; (0/w) IRQ1 interrupt acknowledge flag
IMASK1 = 0x02 ; IRQ1 interrupt mask
MODE1  = 0x01 ; IRQ1 edge select (0=falling and low, 1=falling)


KBSCR  = 0x1b ; Keyboard Status/Control

PCTL   = 0x1c ; Phase-Loked Loop (PLL) Control Reg.

PLLIE  = 0x80 ; PLL interrupt enable bit
PLLF   = 0x40 ; (w) PLL interrupt flag
PLLON  = 0x20 ; PLL on bit
BCS    = 0x10 ; Base clock select bit

PBWC = 0x1d       ; PLL Bandwidth Control Reg.

AUTOBAND  = 0x80 ; Automatic bandwidth control bit
LOCK      = 0x40 ; (r) Lock indicator bit
ACQN      = 0x20 ; Acquisition mode bit
XLD       = 0x10 ; Crystal loss detect bit

PPG = 0x1e      ; PLL Programming Reg.

PPG_MUL_1  = 0x10  ; VCO Frequency Multiplier 1
PPG_MUL_2  = 0x20  ; VCO Frequency Multiplier 2
PPG_MUL_3  = 0x30  ; VCO Frequency Multiplier 3
PPG_MUL_4  = 0x40  ; VCO Frequency Multiplier 4
PPG_MUL_5  = 0x50  ; VCO Frequency Multiplier 5
PPG_MUL_6  = 0x60  ; VCO Frequency Multiplier 6
PPG_MUL_7  = 0x70  ; VCO Frequency Multiplier 7
PPG_MUL_8  = 0x80  ; VCO Frequency Multiplier 8
PPG_MUL_9  = 0x90  ; VCO Frequency Multiplier 9
PPG_MUL_10 = 0xa0  ; VCO Frequency Multiplier 10
PPG_MUL_11 = 0xb0  ; VCO Frequency Multiplier 11
PPG_MUL_12 = 0xc0  ; VCO Frequency Multiplier 12
PPG_MUL_13 = 0xd0  ; VCO Frequency Multiplier 13
PPG_MUL_14 = 0xe0  ; VCO Frequency Multiplier 14
PPG_MUL_15 = 0xf0  ; VCO Frequency Multiplier 15

TSCR = 0x20      ; Timer Status & Control Register

TDMA   = 0x21    ; Timer DMA Select Register
TCNTH  = 0x22    ; Timer Counter Register Hi
TMCNTL = 0x23    ; Timer Counter Register Lo
TMODH  = 0x24    ; Timer Counter Modulo Register Hi
TMODL  = 0x25    ; Timer Counter Modulo Register Lo
TSCR0  = 0x26    ; Timer Channel 0 Status & Control Register
CH0RH  = 0x27    ; Timer Channel 0 Register Hi
CH0RL  = 0x28    ; Timer Channel 0 Register Lo
TSCR1  = 0x29    ; Timer Channel 1 Status & Control Register
CH1RH  = 0x2a    ; Timer Channel 1 Register Hi
CH1RL  = 0x2b    ; Timer Channel 1 Register Lo
TSCR2  = 0x2c    ; Timer Channel 2 Status & Control Register
CH2RH  = 0x2d    ; Timer Channel 2 Register Hi
CH2RL  = 0x2e    ; Timer Channel 2 Register Lo
TSCR3  = 0x2f    ; Timer Channel 3 Status & Control Register
CH3RH  = 0x30    ; Timer Channel 3 Register Hi
CH3RL  = 0x31    ; Timer Channel 3 Register Lo

COPCTL = 0xffff   ; Computer Operates Properly Control register

.if STANDALONE
        .area _STACK
stack_length = 0x40
stack:  .ds stack_length

        .area _CODE  ; put program code into its place
        jmp start
.endif

        .area _CODE  ; put program code into its place

t_crlf:  .asciz '\r\n'

t_brk:   .asciz '\r\nBrk:'

t_call:  .asciz '\r\nMon:'

t_badcom:.asciz 'Bad command!\r\n'

t_dhead: .asciz 'Addr  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F\r\n'

.if FULL_SIZE
 t_cok:   .asciz 'Compare O.K.\r\n'
 
 t_cbad1: .asciz 'Difference -> '
		 
t_snot:  .asciz '\r\nData not found !\r\n'

t_sfound:.asciz '\r\nData found at '
.endif

t_SP:    .asciz ' SP='
t_hx:    .asciz ' HX='
t_pc:    .asciz ' PC='
t_a:     .asciz ' A='
t_ccr:   .asciz ' CCR='

t_CY:    .asciz ' CY'
t_Z:     .asciz ' Z'
t_N:     .asciz ' N'
t_I:     .asciz ' I'
t_HC:    .asciz ' HCY'
t_OV:    .asciz ' OV'


.if FULL_SIZE
;-------------------------------------
; LEDon
;-------------------------------------
LEDon:   sta 0xffff
         bset #2,DDRE
         bclr #2,PTE
         rts

;-------------------------------------
;  LEDoff
;-------------------------------------
LEDoff:  sta 0xffff
         bset #2,DDRE
         bset #2,PTE
         rts

;-------------------------------------
;  delay
;-------------------------------------
delay:    cphx #0
          beq  delay_e
          aix  #-1

          pshx
          pshh
          ldhx #0x0400

delay_tc: sta 0xffff

          cphx #0
          beq  delay_te
          aix  #-1
          bra delay_tc

delay_te: pulh
          pulx
          bra  delay

delay_e:  rts


;-------------------------------------
;  blink
;-------------------------------------
time_slice = 0x0050

blink3:  lda #3
blink:   pshx
         pshh

blink_c: bsr LEDon

         ldhx #time_slice
         bsr delay

         bsr LEDoff

         psha
         lda #'A'
         jsr wrtchr
         pula

         ldhx #time_slice
         bsr delay

         dbnza  blink_c

         ldhx #(time_slice*4)
         bsr delay

         pulh
         pulx
         rts
.endif

.if STANDALONE
start:   sei
         ldhx #stack+stack_length-1 ; initial stack pointer
         txs

         ; Init Phase-Locked Loop
         lda  #0x00          ; turn off PLL
         sta  PCTL
         lda  #0x44          ; sel 8 Mhz bus clk if F_crystal=4,9152 MHz
         sta  PPG
         lda  #AUTO          ; sel auto tracking mode
         sta  PBWC
         lda  #PLLON         ; turn on PLL
         sta  PCTL

wait_lock:                   ; wait for PLL Lock
         lda PBWC
         and #LOCK
         beq wait_lock

         lda #PLLON|BCS      ; switch over to PLL frequency
         sta PCTL


	 .if IO_channel-SCI_channel
;   Initialize the SCI.
;
        ;mov #init_scbr,scbr   ; initialize baud rate
        ;mov #init_scc1,scc1   ; initialize SCI Control Register 1
        ;mov #init_scc2,scc2   ; initialize SCI Control Register 2
init_scc1 = 0b01000000      ; enable SCI, 8-bits, no par, 1 stop
init_scc2 = (TE + RE)       ; no interupts, rcvr and tmtr enabled
init_scbr = 0b00000000      ; = SCI for 115.2 kbaud

          mov #init_scc1,scc1   ; initialize SCI Control Register 1
          mov #init_scc2,scc2   ; initialize SCI Control Register 2
;
	.endif
         cli

cik:     jsr monitor
         nop
         nop
         nop
         ldhx #t_sfound
         jsr wrttxt
         nop

         bra cik

.endif

;-------------------------------------
;  SWI IT - caused by a break IT
;-------------------------------------

SWI_IT:: pshh               ; H,CCR,A,X,PCH,PCL

         bra   MoniSWI

;-------------------------------------
;  monitor - a monitor szolgltatsai
;-------------------------------------
_Monitor:: 
monitor:: pshx               ; create an IT frame
         psha               ; H,CCR,A,X,PCH,PCL
         tpa
         psha
         pshh
         sei                ; disable interrupts

MoniSWI: sta   0xFFFF
         pula               ; save status in RAM variables
         sta   s_H
         pula
         sta   s_CCR
         pula
         sta   s_A
         pula
         sta   s_X
         pula
         sta   s_PCH
         sta   m_cim         ; start unassemble here
         pula
         sta   s_PCL
         sta   m_cim+1

         tsx
         sthx  s_SP          ; stack before call or break

         ; check if it is a break IT

         lda   BRKSCR
         BIT   #0x40
         beq   callir       ; monitor call

         clra               ; disable IT
         sta   BRKSCR

         lda   #BFCE        ; Break Celar flag enable bit
         sta   SBFCR        ; SIM Break Flag Control Register

         ldhx  #t_brk       ; Let to know that it is break point
         bra   calir1

callir:  clra               ; disable IT
         sta   BRKSCR

         lda   #BFCE        ; Break Celar flag enable bit
         sta   SBFCR        ; SIM Break Flag Control Register

         ldhx  #t_call      ; Let to know that it is monitor call
calir1:  jsr   wrttxt


         ; EXIT to comm_po

;--------------------------------;
; . - write out register status  ;
;--------------------------------;

; PC=0000 [nop] A=00 H:X=0000 SP=0000 CCR=00 [ OV HCY I N Z CY]

comm_po: ldhx  #t_pc       ; PC=xxxx
         jsr   wrttxt
         ldhx  s_PC
         jsr   wrtword

         jsr   wrtsp

 .if FULL_SIZE
         lda   #'['
         jsr   wrtchr

         ldhx  s_PC
         jsr   u_hx

         lda   #']'
         jsr   wrtchr
         jsr   wrtsp
.else
	lda   #'['
	jsr   wrtchr
	
	ldhx  s_PC
	lda   ,x
	jsr   wrtbyte
	ldhx  s_PC
	lda   1,x
	jsr   wrtbyte
	ldhx  s_PC
	lda   2,x
	jsr   wrtbyte
	
	lda   #']'
	jsr   wrtchr
	jsr   wrtsp

.endif

         ldhx  #t_a        ; A=xx
         jsr   wrttxt
         lda   s_A
         jsr   wrtbyte

         ldhx  #t_hx       ; HX=xxxx
         jsr   wrttxt
         ldhx  s_HX
         jsr   wrtword

         ldhx  #t_sp       ; SP=xxxx
         jsr   wrttxt
         ldhx  s_SP
         jsr   wrtword

         ldhx  #t_ccr      ; CCR=xx
         jsr   wrttxt
         lda   s_CCR
         jsr   wrtbyte

         ; write CCR details
         jsr   wrtsp
         lda   #'['
         jsr   wrtchr

         brclr #7,*s_CCR,stat1        ; OV
         ldhx  #t_ov
         jsr   wrttxt

stat1:   brclr #4,*s_CCR,stat2        ; HC
         ldhx  #t_hc
         jsr   wrttxt

stat2:   brclr #3,*s_CCR,stat3        ; I
         ldhx  #t_I
         jsr   wrttxt

stat3:   brclr #2,*s_CCR,stat4        ; N
         ldhx  #t_N
         jsr   wrttxt

stat4:   brclr #1,*s_CCR,stat5        ; Z
         ldhx  #t_Z
         jsr   wrttxt


stat5:   brclr #0,*s_CCR,stat6        ; CY
         ldhx  #t_CY
         jsr   wrttxt

stat6:   lda   #']'
         jsr   wrtchr
         jsr   wrtcrlf

         bra   command


badcomm: ldhx   #t_badcom    ; bad comand
         jsr    wrttxt


Command: ; main command executing loop of monitor
         lda    #'-'         ; write prompt
         jsr    wrtchr

         jsr    getln        ; read command line

         jsr    spaces       ; read over traling spaces

         jsr    lgetch       ; read first char of line

         jsr    to_upper

         cmp    #'D'
         beq    i_com_d

         cmp    #'E'
         beq    i_com_e

.if FULL_SIZE
         cmp    #'F'
         beq    i_com_f
.endif

         cmp    #'G'
         beq    i_com_g

 .if FULL_SIZE
         cmp    #'H'
         beq    i_com_h

         cmp    #'?'
         beq    i_com_h

         cmp    #'M'
         beq    i_com_m

         cmp    #'S'
         beq    i_com_s

         cmp    #'U'
         beq    i_com_u
.endif

         cmp    #'.'
         beq    i_com_po

         cmp    #'N'
         beq    i_com_n

         cmp    #'O'
         beq    i_com_o

.if FULL_SIZE
         cmp    #'C'
         beq    comm_c
.endif
         jmp    badcomm

i_com_d: jmp    comm_d
i_com_e: jmp    comm_e
.if FULL_SIZE
i_com_f: jmp    comm_f
.endif
i_com_g: jmp    comm_g
.if FULL_SIZE
i_com_h: jmp	comm_h
i_com_m: jmp    comm_m
.endif
i_com_n: jmp    comm_n
i_com_o: jmp    comm_o
.if FULL_SIZE
i_com_s: jmp    comm_s
i_com_u: jmp    comm_u
.endif
i_com_po:jmp    comm_po

.if FULL_SIZE
;----------------------------------------------------------
;    c  range address  - compare data field
;----------------------------------------------------------

comm_c:; read parameters
       jsr   rdxword         ; 1-st parameter
       bcs   c_0             ; sytax error
       sthx  m_beg

       jsr   rdxword         ; 2-nd parameter
       bcs   c_0             ; sytax error
       sthx  m_end

       jsr   rdxword         ; 3-nd parameter
       bcs   c_0             ; sytax error
       sthx  m_ind

       jsr   spaces
       jsr   lpeekch
       cmp    #'\r'          ; good parameters
       beq    c_1
c_0:   jmp   badcomm


c_1:   ; compare BEG pointer to end pointer
       ldhx  m_beg
       cphx  m_end
       bls   c_3        ; compare next data

       ; end of field, compare success
       ldhx  #t_cok
       jsr   wrttxt
       jmp   command

c_3:   ;compare next data byte
       sta   0xFFFF

       ldhx  m_beg
       lda   ,x
       ldhx  m_ind
       cmp   ,x
       beq   c_4             ; the same

       ldhx  #t_cbad1        ; Message: Difference -> A086:44 B086:55
       jsr   wrttxt

       ldhx  m_beg
       jsr   wrtword

       lda   #':'
       jsr   wrtchr

       ldhx  m_beg
       lda   ,x
       jsr   wrtbyte

       jsr   wrtsp

       ldhx  m_ind
       jsr   wrtword

       lda    #':'
       jsr   wrtchr

       ldhx  m_ind
       lda   ,x
       jsr   wrtbyte

       jsr   wrtcrlf
       jmp  command

c_4:   ldhx  m_beg           ; increment pointers
       aix   #+1
       sthx  m_beg

       ldhx  m_ind
       aix   #+1
       sthx  m_ind
       bra   c_1
.endif

;----------------------------------------------------------
;  d  [address] | [range]  - dump from RAM
;----------------------------------------------------------

comm_d:jsr   spaces
       jsr   lpeekch
.if FULL_SIZE
       cmp   #'.'
       bne   .+5
       jmp   d_r         ; dump register
.endif
       cmp   #'\r'
       bne   .+5
       jmp   d_10         ; default dump

       jsr   rdxword     ; 1-st parameter
       bcs   d_0         ; sytax error
       sthx  m_beg

       jsr   spaces
       jsr   lpeekch
       cmp   #'\r'
       beq   d_100       ; default area end

       jsr   rdxword     ; 2-nd parameter
       bcs   d_0         ; sytax error
       sthx  m_end

       jsr   spaces
       jsr   lpeekch
       cmp   #'\r'
       beq   d_11        ; dump by parameters

d_0:   jmp   badcomm


       ; create defult values
d_10:  ldhx  m_cim      ; start address
       sthx  m_beg

d_100: ldhx  m_beg
       aix   #+0x3f      ; end=beg+$3f
       sthx  m_end

d_11:  ldhx #t_dhead    ; write title line
       jsr   wrttxt

       ldhx  m_beg      ; set row address onto 16 byte bound
       txa
       and   #0xf0
       tax
       sthx  m_ind

d_2:   ; write out address on 16-byte bound
       lda    m_ind+1
       and    #0x0f
       bne    d_3       ; not on bound

       jsr   wrtcrlf    ; Write row address
       lda   m_ind
       jsr   wrtbyte
       lda   m_ind+1
       and   #0xf0
       jsr   wrtbyte

d_3:   ; write out data
       jsr  wrtsp       ; traling space

       ; when the pointer is lower then the beg it writes out gap
       ldhx  m_beg
       cphx  m_ind
       bls   d_4        ; write data

       lda    #'.'      ; write empty gap
       jsr   wrtchr
       lda    #'.'
       jsr   wrtchr
       bra   d_5

d_4:   ; write data
       ldhx  m_ind
       lda   ,x
       jsr   wrtbyte

d_5:   ; increment data pointer
       ldhx  m_ind
       aix   #+1
       sthx  m_ind

       ldhx  m_ind      ; check end limit
       cphx  m_end
       bls   d_2        ; still write data

       jsr   wrtcrlf    ; finist last line

       ldhx   m_ind     ; save end address as next default
       sthx   m_cim

       jmp   command

.if FULL_SIZE
;.....................................................
; command with register name
d_re:     jmp   badcomm

d_r:      jsr   findreg      ; find register (addr->m_beg)
          bcs   d_re         ; could not find it
          jsr   spaces       ; read data byte
          jsr   lpeekch
          cmp   #'\r'
          bne   d_re

.endif
d_rd:     ldhx  m_beg        ; write out data
          lda   ,x
          jsr   wrtbyte

          jsr   wrtcrlf      ; finish line
          jmp   command


;----------------------------------------------------------
;  e  address list       - enter data into RAM
;  e  .reg byte          - enter data into register
;----------------------------------------------------------

e_0:      jmp   badcomm
e_1:      jmp   command

comm_e:   jsr   spaces
          jsr   lpeekch
.if FULL_SIZE
          cmp   #'.'
          beq   e_r          ; register use
.endif
          jsr   rdxword      ; address parameter
          bcs   e_0          ; sytax error
          sthx  m_beg

e_2:      jsr   spaces
          jsr   lpeekch
          cmp   #'\r'
          beq   e_1          ; end of line

          jsr   rdxword      ; data parameter
          bcs   e_0          ; sytax error
          txa                ; put data into A

          ldhx  m_beg        ; put data into memory
          sta   ,x

e_4:      ldhx  m_beg        ; increment address
          aix   #+1
          sthx  m_beg

          bra   e_2          ; next data on line


.if FULL_SIZE
; ....................................................
; comand with register name
e_r:      jsr   findreg     ; find register
          bcs   e_re        ; could not find

          jsr   spaces      ; read data byte
          jsr   rdxword
          bcs   e_re        ; bad data syntax
          txa               ; put data byte into A

          ldhx  m_beg       ; write out data
          sta   ,x
          jmp   command

e_re:     jmp   badcomm
.endif


.if FULL_SIZE
;--------------------------------------------------
;  f   range  list       - fill data field in ram
;--------------------------------------------------

f_0:   jmp    badcomm

comm_f:jsr   rdxword     ; 1-st parameter
       bcs   f_0         ; sytax error
       sthx  m_beg

       jsr   rdxword     ; 2-nd parameter
       bcs   f_0         ; sytax error
       sthx  m_end

       clr   m_leng      ; restart data list

f_1:   jsr   spaces
       jsr   lpeekch
       cmp   #'\r'
       beq   f_2         ; fill from the start of data list

       jsr    rdxword    ; read next data byte
       bcs    f_0        ; syntax error
       txa               ; put data into A

       clrh              ; put data into list
       ldx    m_leng
       sta    m_list,x

       inc    m_leng     ; increment list length

       bra    f_1        ; try to read the next byte on line

; data in list, now fill memory
f_2:   clr   m_ind       ; reset list index

f_3:
       sta   0xFFFF

       ldhx  m_beg       ; compare area limits
       cphx  m_end
       bls   f_4         ; fill with data
       jmp   command     ; area filled

f_4:   clrh              ; read next byte from list
       ldx   m_ind
       lda   m_list,x

       inc   m_ind       ; increment index
       ldx   m_ind       ; end of list ?
       cpx   m_leng
       bne   f_5         ; no

       clr   m_ind       ; start from the first item in list

f_5:   ldhx  m_beg       ; save data into memory
       sta   ,x

       aix   #+1         ; increment data pointer
       sthx  m_beg

       bra   f_3
.endif

;------------------------------------------------
;  o - step an instruction, over jsr and bsr
;------------------------------------------------
comm_o: ldhx  s_PCH      ; Get actual program counter
        lda ,x
        cmp #0xAD        ; bsr rel
        beq comm_o2
        cmp #0xBD        ; jsr DIR
        beq comm_o2
        cmp #0xCD        ; jsr EXT
        beq comm_o3
        cmp #0xDD        ; jsr IX2
        beq comm_o3
        cmp #0xED        ; jsr IX1
        beq comm_o2
        cmp #0xFD        ; jsr IX
        beq comm_o1
        bra comm_n

comm_o3: aix #1
comm_o2: aix #1
comm_o1: aix #1
         bra  go_hx

;------------------------------------------------
;  n - step an instruction
;------------------------------------------------
; - break is after next instruction of any length
;   if trap is set after the first byte
; - in case of [jmp/jsr ,x] trap is set to instuction
;   pointed by H:X

comm_n: ldhx  s_PCH      ; Get actual program counter
        lda ,x
        aix #1           ; Check if instruction is a jmp ,x or jsr ,x
        and #0xFE
        cmp #0xFC
        bne go_hx

        lda  s_H         ; Get H:X
        psha
        pulh
        ldx  s_X

        lda ,x           ; Get [H:X]
        ldx 1,x          ; Then = break at address pointed to by x-reg
        psha
        pulh
        bra  go_hx


;------------------------------------------------
;  g[m] [=start] [stop]  - go from start to stop
;------------------------------------------------

go_bad:jmp  badcomm

comm_g:jsr    spaces
       jsr    lpeekch
       cmp    #'\r'
       beq    go_on
       cmp    #'='
       beq    go_from
       bra    go_to

; run from a given address
go_from:jsr   lgetch    ; read over =
        jsr   rdxword   ; read stop parameter
        bcs   go_bad    ; bad syntax
        sthx  s_PC      ; store the start address

       jsr   spaces     ; read over spaces at line end
       jsr   lpeekch    ; What is after start address
       cmp   #'\r'
       beq   go_on      ; start without stop address

; stop address given
go_to: jsr   rdxword     ; read stop address
       bcs   go_bad      ; syntax error

go_hx: clra              ; disable IT
       sta   BRKSCR

       stx   BRKL        ; store the stop address
       pshh
       pula
       sta   BRKH

       jsr   spaces      ; read over spaces at line end
       jsr   lpeekch
       cmp   #'\r'
       bne   go_bad      ; must be line end

       lda  #0x80        ; enable IT
       sta  BRKSCR

       lda   BRKH
       ldx   BRKL
       psha
       pulh
       bra go_on1


; restore status data for user level
go_on:  ldhx s_PC
        jsr wrtword

go_on1: lda  s_PCL
        psha
        lda  s_PCH
        psha
        lda  s_X
        psha
        lda  s_A
        psha
        lda  s_CCR
        psha
        lda  s_H
        psha
        pulh
        rti              ; restores also IT mask

.if FULL_SIZE
;-----------------------------------------------------
;    m  range address      - move data field in ram
;-----------------------------------------------------

comm_m:jsr   rdxword     ; 1-st parameter
       bcs   m_0         ; syntax error
       sthx  m_beg

       jsr   rdxword     ; 2-nd parameter
       bcs   m_0         ; syntax error
       sthx  m_end

       jsr   rdxword     ; 3-rd parameter
       bcs   m_0         ; syntax error
       sthx  m_ind

       jsr   spaces
       jsr   lpeekch
       cmp   #'\r'
       beq   m_1         ; good parameters

m_0:   jmp  badcomm

m_1:   sta   0xFFFF

       ldhx  m_beg       ; check area limits
       cphx  m_end
       bls   m_3         ; copy data
       jmp   command     ; end of area

m_3:   ldhx  m_beg       ; copy data
       lda   ,x
       aix   #+1
       sthx  m_beg

       ldhx  m_ind
       sta   ,x
       aix   #+1
       sthx  m_ind

       bra   m_1
.endif

;-------------------------------------------;
; hexdig - convert ASCII to hex digit       ;
; i: a - ASCII char                         ;
; o: c  =0 hexa digit  (A=digit bin value)  ;
;       =1 no conversion                    ;
;-------------------------------------------;
hexdig:   cmp  #'0'
          blo   hexdie  ; less then 0

          cmp  #'9'
          bhi  hexdi1   ;  greather then '9'

          sub  #'0'     ; number
          clc
          rts

hexdi1:   cmp  #'A'
          blo  hexdie   ; less then 'A'

          cmp  #'F'
          bhi  hexdi2   ; greather then 'F'

          sub  #'A'-10  ; A..F
          clc
          rts

hexdi2:   cmp  #'a'
          blo  hexdie   ; less then 'a'

          cmp  #'f'
          bhi  hexdie   ; greather then 'f'

          sub  #'a'-10  ; a..f
          clc
          rts

hexdie:   sec
          rts


.if FULL_SIZE
;--------------------------------------------------
;  s   range  list       - search data field in ram
;--------------------------------------------------

s_0:   jmp    badcomm

comm_s:jsr   rdxword     ; 1-st parameter
       bcs   s_0         ; sytax error
       sthx  m_beg

       jsr   rdxword     ; 2-nd parameter
       bcs   s_0         ; sytax error
       sthx  m_end

; store data list in byte format

       clr   m_leng      ; empty data list

s_1:   jsr   spaces
       jsr   lpeekch
       cmp    #'\r'
       beq    s_2        ; fill from the start of data list

       jsr    rdxword    ; read next data byte
       bcs    s_0        ; syntax error
       txa               ; put data into A

       clrh              ; put data into list
       ldx    m_leng
       sta    m_list,x

       inc    m_leng     ; increment list length
       bra    s_1        ; try to read the next byte on line


; match memory content with data list
s_2:   sta   0xffff

       ldhx  m_beg       ; compare area limits
       cphx  m_end
       bls   s_6         ; find data

s_4:   ldhx  #t_snot    ; Not found
       jsr   wrttxt
s_5:   jmp   command


; search list
s_6:   clr   m_ind       ; reset list inde

       ldhx   m_beg      ; remember actual start of search
       sthx   m_pos

; for all data byte
s_7:   lda    m_ind      ; end of list reached ?
       cmp    m_leng
       bne    s_8        ; not yet

       ; All the bytes in the list compared, that is success
       ldhx  #t_sfound
       jsr   wrttxt
       ldhx  m_pos
       jsr   wrtword
       jsr   wrtcrlf
       bra   s_5

s_8:   clrh              ; compare actual byte
       ldx   m_ind
       lda   m_list,x

       ldhx  m_beg
       cmp   ,x
       bnz   s_9         ; diffrence

       aix   #+1         ; increment data pointer
       sthx  m_beg

       cphx  m_end
       bhi   s_4         ; area ended

       inc   m_ind
       bra   s_7

s_9:   ; difference try to match list from the next RAM byte
       ldhx  m_pos    ; increment data addres
       aix   #+1
       sthx  m_beg
       bra   s_2
.endif

.if FULL_SIZE

;----------------------------------------------
;  u  range   - unassembl statements
;----------------------------------------------

jmptab:.dw 0
       .dw form1
       .dw form2
       .dw form3
       .dw form4
       .dw form5
       .dw form6
       .dw form7
       .dw form8
       .dw form9
       .dw form10
       .dw form11
       .dw form12
       .dw form13
       .dw form14
       .dw form15
       .dw form16
       .dw form17
       .dw form18
       .dw form19
       .dw form20
       .dw form21
       .dw form22
       .dw form23
       .dw form24



u_0:   jmp badcomm

comm_u:jsr   spaces
       jsr   lpeekch
       cmp    #'\r'
       beq    u_1        ; default unassembly

       jsr   rdxword     ; 1-st parameter
       bcs   u_0         ; sytax error
       sthx  m_cim

       jsr   spaces      ; must be line end
       jsr   lpeekch
       cmp   #'\r'
       bne   u_0

u_1:   clr   m_db        ; clear line counter

u_2:   lda   m_db        ; stop after 10 lines
       cmp   #10
       blt   u_3         ; list one more line
       jmp   command     ; command finished

u_3:   ldhx  m_cim       ; write out address
       jsr   wrtword

       jsr   wrtsp

u_sp:  ldhx  m_cim       ; create mnemonic from opcode

       jsr   u_hx        ; disassemble instruction, return length
       tsta
       bmi   u_sp        ; SP group

u_4:   add   m_cim+1     ; step over statement , length in A
       sta   m_cim+1
       bcc   u_5
       inc   m_cim

u_5:   jsr   wrtcrlf     ; terminate the line

       inc   m_db        ; increment line count
       jmp   u_2         ; new line


;----------------------------------------------
;  u_hx  - disassemble instruction
; I: H:X - address of inst.
; O: A - instr. length (-1 for SP group)
;----------------------------------------------

u_hx:  lda   ,x

       ldx   #3
       mul               ; X:A = opcode * 3

       add   #<stmtab
       psha
       txa
       adc   #>stmtab
       psha
       pulh
       pulx              ; H:X = stmtab + opcode*4

       pshx              ; save table pointer
       pshh

       lda   1,x         ; get mnemonic address
       ldx   2,x
       psha
       pulh

       jsr   wrttxt      ; write out mnemonic
       jsr   wrtsp

       pulh              ; restore table pointer
       pulx

       lda   ,x          ; jump to format routine
       ldx   #2
       mul
       pshx
       pulh
       tax
       lda   jmptab,x
       ldx   jmptab+1,x
       psha
       pulh
       jmp   ,x


inc_m_cim:
       ldhx m_cim
       aix  #+1
       sthx m_cim
       rts

lcimx1w: ldhx m_cim
         lda  1,x
         jmp  wrtbyte


form1: lda  #1        ; psha
       rts

form2: lda   #'='     ; lda =c1
       jsr  wrtchr

form3: ldhx m_cim     ; lda a1
       ldx  1,x
       clrh
       jsr  wrtword
       lda  #2
       rts


form4: lda   #'='     ; ldhx =c2
       jsr  wrtchr

form5: ldhx m_cim     ; lda a2
       lda  1,x
       ldx  2,x
       psha
       pulh
       jsr  wrtword

       lda  #3
       rts

form6: ldhx m_cim     ; brset 0,a1,r1
       lda  ,x
       lsra
       and   #7

form6i:jsr  wrtbyte

       jsr  wrtcomma
       bra  form11    ; for 11 does the rest


form7: ldhx m_cim     ; brclr 0,a1,r1
       lda  ,x
       lsra
       and  #0x7
       bra  form6i


form8: ldhx m_cim     ; bset 0,a1
       lda  ,x
       lsra
       and  #0x7

form8i:jsr  wrtbyte

       jsr  wrtcomma

       jsr  lcimx1w

       lda  #2
       rts

form9: ldhx m_cim     ; bclr 0,a1
       lda  ,x
       lsra
       and  #0x7
       bra  form8i

form11:jsr  lcimx1w  ; cbeq a1,r1

       jsr  wrtcomma

       jsr  inc_m_cim

form10:ldhx m_cim     ; bra a1
       lda  1,x       ; calculate absolute address
       psha
       bmi  f10_1
       clra
       bra  f10_2
f10_1: lda #0xFF
f10_2: psha

       lda  m_cim+1
       add  2,sp
       sta  2,sp
       lda  m_cim
       adc  1,sp
       sta  1,sp
       pulh
       pulx
       aix  #+2
       jsr  wrtword

       lda  #2
       rts

form12:jsr  lcimx1w  ; lda o1,x

       jsr  wrtcomma

       ldhx m_cim

       aix  #-1
       lda  ,x

       cmp  #0x9e
       beq  f12_1

       lda  #'x'
       bra  f12_2

f12_1: lda  #'s'
       jsr  wrtchr
       lda  #'p'
f12_2: jsr  wrtchr

       lda  #2
       rts

form13:jsr  wrtcomma    ; lda ,x
       lda  #'x'
       jsr  wrtchr
       lda  #1
       rts

form14:lda  #'='       ; cbeqa =c1,r1
       jsr  wrtchr

       jsr  lcimx1w
       jsr  wrtcomma

       jsr  inc_m_cim
       bra  form10

form15:jsr  lcimx1w    ; move a1,a1

       jsr  wrtcomma

       ldhx m_cim
       lda  2,x
       jsr  wrtbyte

       lda  #3
       rts


form16:jsr  lcimx1w     ; mov a1,x+

       jsr  wrtcomma

       lda  #'x'
       jsr  wrtchr
       lda  #'+'
       jsr  wrtchr
       lda  #2
       rts

form17:jsr  lcimx1w     ; cbeq a1,x+,r1

       jsr  wrtcomma

       ldhx m_cim
       aix  #-1
       lda  ,x
       cmp  #0x9e
       beq  f17_1

       lda  #'x'
       jsr  wrtchr
       lda  #'+'
       bra  f17_2

f17_1: lda  #'s'
       jsr  wrtchr
       lda  #'p'
f17_2: jsr  wrtchr

       jsr  wrtcomma

       jsr  inc_m_cim
       jmp  form10

form18:jsr  lcimx1w     ; dbnz a1,x,r1

       jsr  wrtcomma

       ldhx m_cim
       aix  #-1
       lda  ,x
       cmp  #0x9e
       beq  f18_1

       lda  #'x'
       bra  f18_2

f18_1: lda  #'s'
       jsr  wrtchr
       lda  #'p'
f18_2: jsr  wrtchr

       jsr  wrtcomma

       jsr  inc_m_cim
       jmp  form10


form19:lda   #'='     ; mov =c1,a1
       jsr  wrtchr

       jsr  lcimx1w   ; lda a2
       jsr  wrtcomma
       ldhx m_cim
       lda  2,x
       jsr  wrtbyte

       lda  #3
       rts

form20:lda  #'x'      ; cbeq x+,r1
       jsr  wrtchr
       lda  #'+'
f20_1: jsr  wrtchr
       jsr  wrtcomma

       jmp  form10

form21:lda  #'x'      ; dbnz x,r1
       bra  f20_1

form22:lda  #'x'      ; cbeq x+,r1
       jsr  wrtchr
       lda  #'+'
       jsr  wrtchr
       jsr  wrtcomma

       jsr  lcimx1w

       lda  #2
       rts

; prepares SP group
form23:jsr  inc_m_cim

       lda  #'\b'
       jsr  wrtchr
       lda  #-1
       rts

form24:ldhx m_cim     ; lda o2,x
       lda  1,x
       ldx  2,x
       psha
       pulh
       jsr  wrtword

       jsr  wrtcomma

       ldhx m_cim
       aix  #-1
       lda  ,x
       cmp  #0x9e
       beq  f24_1

       lda  #'x'
       bra  f24_2

f24_1: lda  #'s'
       jsr  wrtchr
       lda  #'p'
f24_2: jsr  wrtchr

       lda  #3
       rts


stmtab:
       .byte 6,>__brset,<__brset      ; 0x00
       .byte 7,>__brclr,<__brclr      ; 0x01
       .byte 6,>__brset,<__brset      ; 0x02
       .byte 7,>__brclr,<__brclr      ; 0x03
       .byte 6,>__brset,<__brset      ; 0x04
       .byte 7,>__brclr,<__brclr      ; 0x05
       .byte 6,>__brset,<__brset      ; 0x06
       .byte 7,>__brclr,<__brclr      ; 0x07
       .byte 6,>__brset,<__brset      ; 0x08
       .byte 7,>__brclr,<__brclr      ; 0x09
       .byte 6,>__brset,<__brset      ; 0x0a
       .byte 7,>__brclr,<__brclr      ; 0x0b
       .byte 6,>__brset,<__brset      ; 0x0c
       .byte 7,>__brclr,<__brclr      ; 0x0d
       .byte 6,>__brset,<__brset      ; 0x0e
       .byte 7,>__brclr,<__brclr      ; 0x0f
       .byte 8,>__bset,<__bset        ; 0x10,
       .byte 9,>__bclr,<__bclr        ; 0x11,
       .byte 8,>__bset,<__bset        ; 0x12,
       .byte 9,>__bclr,<__bclr        ; 0x13,
       .byte 8,>__bset,<__bset        ; 0x14,
       .byte 9,>__bclr,<__bclr        ; 0x15,
       .byte 8,>__bset,<__bset        ; 0x16,
       .byte 9,>__bclr,<__bclr        ; 0x17,
       .byte 8,>__bset,<__bset        ; 0x18,
       .byte 9,>__bclr,<__bclr        ; 0x19,
       .byte 8,>__bset,<__bset        ; 0x1a,
       .byte 9,>__bclr,<__bclr        ; 0x1b,
       .byte 8,>__bset,<__bset        ; 0x1c,
       .byte 9,>__bclr,<__bclr        ; 0x1d,
       .byte 8,>__bset,<__bset        ; 0x1e,
       .byte 9,>__bclr,<__bclr        ; 0x1f,
       .byte 10,>__bra,<__bra         ; 0x20,
       .byte 10,>__brn,<__brn         ; 0x21,
       .byte 10,>__bhi,<__bhi         ; 0x22,
       .byte 10,>__bls,<__bls         ; 0x23,
       .byte 10,>__bhs,<__bhs         ; 0x24,
       .byte 10,>__blo,<__blo         ; 0x25,
       .byte 10,>__bne,<__bne         ; 0x26,
       .byte 10,>__beq,<__beq         ; 0x27,
       .byte 10,>__bhcc,<__bhcc       ; 0x28,
       .byte 10,>__bhcs,<__bhcs       ; 0x29,
       .byte 10,>__bpl,<__bpl         ; 0x2a,
       .byte 10,>__bmi,<__bmi         ; 0x2b,
       .byte 10,>__bmc,<__bmc         ; 0x2c,
       .byte 10,>__bms,<__bms         ; 0x2d,
       .byte 10,>__bil,<__bil         ; 0x2e,
       .byte 10,>__bih,<__bih         ; 0x2f,
       .byte 3,>__neg,<__neg          ; 0x30,
       .byte 11,>__cbeq,<__cbeq       ; 0x31,
       .byte 1,>__bad,<__bad          ; 0x32,
       .byte 3,>__com,<__com          ; 0x33,
       .byte 3,>__lsr,<__lsr          ; 0x34,
       .byte 3,>__sthx,<__sthx        ; 0x35,
       .byte 3,>__ror,<__ror          ; 0x36,
       .byte 3,>__asr,<__asr          ; 0x37,
       .byte 3,>__lsl,<__lsl          ; 0x38,
       .byte 3,>__rol,<__rol          ; 0x39,
       .byte 3,>__dec,<__dec          ; 0x3a,
       .byte 11,>__dbnz,<__dbnz       ; 0x3b,
       .byte 3,>__inc,<__inc          ; 0x3c,
       .byte 3,>__tst,<__tst          ; 0x3d,
       .byte 1,>__bad,<__bad          ; 0x3e,
       .byte 3,>__clr,<__clr          ; 0x3f,
       .byte 1,>__nega,<__nega        ; 0x40,
       .byte 14,>__cbeqa,<__cbeqa     ; 0x41,
       .byte 1,>__mul,<__mul          ; 0x42,
       .byte 1,>__coma,<__coma        ; 0x43,
       .byte 1,>__lsra,<__lsra        ; 0x44,
       .byte 4,>__ldhx,<__ldhx        ; 0x45,
       .byte 1,>__rora,<__rora        ; 0x46,
       .byte 1,>__asra,<__asra        ; 0x47,
       .byte 1,>__lsla,<__lsla        ; 0x48,
       .byte 1,>__rola,<__rola        ; 0x49,
       .byte 1,>__deca,<__deca        ; 0x4a,
       .byte 10,>__dbnza,<__dbnza     ; 0x4b,
       .byte 1,>__inca,<__inca        ; 0x4c,
       .byte 1,>__tsta,<__tsta        ; 0x4d,
       .byte 15,>__mov,<__mov         ; 0x4e,
       .byte 1,>__clra,<__clra        ; 0x4f,
       .byte 1,>__negx,<__negx        ; 0x50,
       .byte 14,>__cbeqx,<__cbeqx     ; 0x51,
       .byte 1,>__div,<__div          ; 0x52,
       .byte 1,>__comx,<__comx        ; 0x53,
       .byte 1,>__lsrx,<__lsrx        ; 0x54,
       .byte 3,>__ldhx,<__ldhx        ; 0x55,
       .byte 1,>__rorx,<__rorx        ; 0x56,
       .byte 1,>__asrx,<__asrx        ; 0x57,
       .byte 1,>__lslx,<__lslx        ; 0x58,
       .byte 1,>__rolx,<__rolx        ; 0x59,
       .byte 1,>__decx,<__decx        ; 0x5a,
       .byte 10,>__dbnzx,<__dbnzx     ; 0x5b,
       .byte 1,>__incx,<__incx        ; 0x5c,
       .byte 1,>__tstx,<__tstx        ; 0x5d,
       .byte 16,>__mov,<__mov         ; 0x5e,
       .byte 1,>__clrx,<__clrx        ; 0x5f,
       .byte 12,>__neg,<__neg         ; 0x60,
       .byte 17,>__cbeq,<__cbeq       ; 0x61,
       .byte 1,>__nsa,<__nsa          ; 0x62,
       .byte 12,>__com,<__com         ; 0x63,
       .byte 12,>__lsr,<__lsr         ; 0x64,
       .byte 4,>__cphx,<__cphx        ; 0x65,
       .byte 12,>__ror,<__ror         ; 0x66,
       .byte 12,>__asr,<__asr         ; 0x67,
       .byte 12,>__lsl,<__lsl         ; 0x68,
       .byte 12,>__rol,<__rol         ; 0x69,
       .byte 12,>__dec,<__dec         ; 0x6a,
       .byte 18,>__dbnz,<__dbnz       ; 0x6b,
       .byte 12,>__inc,<__inc         ; 0x6c,
       .byte 12,>__tst,<__tst         ; 0x6d,
       .byte 19,>__mov,<__mov         ; 0x6e,
       .byte 12,>__clr,<__clr         ; 0x6f,
       .byte 13,>__neg,<__neg         ; 0x70,
       .byte 20,>__cbeq,<__cbeq       ; 0x71,
       .byte 1,>__daa,<__daa          ; 0x72,
       .byte 13,>__com,<__com         ; 0x73,
       .byte 13,>__lsr,<__lsr         ; 0x74,
       .byte 3,>__cphx,<__cphx        ; 0x75,
       .byte 13,>__ror,<__ror         ; 0x76,
       .byte 13,>__asr,<__asr         ; 0x77,
       .byte 13,>__lsl,<__lsl         ; 0x78,
       .byte 13,>__rol,<__rol         ; 0x79,
       .byte 13,>__dec,<__dec         ; 0x7a,
       .byte 21,>__dbnz,<__dbnz       ; 0x7b,
       .byte 13,>__inc,<__inc         ; 0x7c,
       .byte 13,>__tst,<__tst         ; 0x7d,
       .byte 22,>__mov,<__mov         ; 0x7e,
       .byte 13,>__clr,<__clr         ; 0x7f,
       .byte 1,>__rti,<__rti          ; 0x80,
       .byte 1,>__rts,<__rts          ; 0x81,
       .byte 1,>__bad,<__bad          ; 0x82,
       .byte 1,>__swi,<__swi          ; 0x83,
       .byte 1,>__tap,<__tap          ; 0x84,
       .byte 1,>__tpa,<__tpa          ; 0x85,
       .byte 1,>__pula,<__pula        ; 0x86,
       .byte 1,>__psha,<__psha        ; 0x87,
       .byte 1,>__pulx,<__pulx        ; 0x88,
       .byte 1,>__pshx,<__pshx        ; 0x89,
       .byte 1,>__pulh,<__pulh        ; 0x8a,
       .byte 1,>__pshh,<__pshh        ; 0x8b,
       .byte 1,>__clrh,<__clrh        ; 0x8c,
       .byte 1,>__bad,<__bad          ; 0x8d,
       .byte 1,>__stop,<__stop        ; 0x8e,
       .byte 1,>__wait,<__wait        ; 0x8f,
       .byte 10,>__bge,<__bge         ; 0x90,
       .byte 10,>__blt,<__blt         ; 0x91,
       .byte 10,>__bgt,<__bgt         ; 0x92,
       .byte 10,>__ble,<__ble         ; 0x93,
       .byte 1,>__txs,<__txs          ; 0x94,
       .byte 1,>__tsx,<__tsx          ; 0x95,
       .byte 1,>__bad,<__bad          ; 0x96,
       .byte 1,>__tax,<__tax          ; 0x97,
       .byte 1,>__clc,<__clc          ; 0x98,
       .byte 1,>__sec,<__sec          ; 0x99,
       .byte 1,>__cli,<__cli          ; 0x9a,
       .byte 1,>__sei,<__sei          ; 0x9b,
       .byte 1,>__rsp,<__rsp          ; 0x9c,
       .byte 1,>__nop,<__nop          ; 0x9d,
       .byte 23,>__,<__               ; sp group; 0x9e,
       .byte 1,>__txa,<__txa          ; 0x9f,
       .byte 2,>__sub,<__sub          ; 0xa0,
       .byte 2,>__cmp,<__cmp          ; 0xa1,
       .byte 2,>__sbc,<__sbc          ; 0xa2,
       .byte 2,>__cpx,<__cpx          ; 0xa3,
       .byte 2,>__and,<__and          ; 0xa4,
       .byte 2,>__bit,<__bit          ; 0xa5,
       .byte 2,>__lda,<__lda          ; 0xa6,
       .byte 2,>__ais,<__ais          ; 0xa7,
       .byte 2,>__eor,<__eor          ; 0xa8,
       .byte 2,>__adc,<__adc          ; 0xa9,
       .byte 2,>__ora,<__ora          ; 0xaa,
       .byte 2,>__add,<__add          ; 0xab,
       .byte 1,>__bad,<__bad          ; 0xac,
       .byte 10,>__bsr,<__bsr         ; 0xad,
       .byte 2,>__ldx,<__ldx          ; 0xae,
       .byte 2,>__aix,<__aix          ; 0xaf,
       .byte 3,>__sub,<__sub          ; 0xb0,
       .byte 3,>__cmp,<__cmp          ; 0xb1,
       .byte 3,>__sbc,<__sbc          ; 0xb2,
       .byte 3,>__cpx,<__cpx          ; 0xb3,
       .byte 3,>__and,<__and          ; 0xb4,
       .byte 3,>__bit,<__bit          ; 0xb5,
       .byte 3,>__lda,<__lda          ; 0xb6,
       .byte 3,>__sta,<__sta          ; 0xb7,
       .byte 3,>__eor,<__eor          ; 0xb8,
       .byte 3,>__adc,<__adc          ; 0xb9,
       .byte 3,>__ora,<__ora          ; 0xba,
       .byte 3,>__add,<__add          ; 0xbb,
       .byte 1,>__bad,<__bad          ; 0xbc,
       .byte 1,>__bad,<__bad          ; 0xbd,
       .byte 3,>__ldx,<__ldx          ; 0xbe,
       .byte 3,>__stx,<__stx          ; 0xbf,
       .byte 5,>__sub,<__sub          ; 0xc0,
       .byte 5,>__cmp,<__cmp          ; 0xc1,
       .byte 5,>__sbc,<__sbc          ; 0xc2,
       .byte 5,>__cpx,<__cpx          ; 0xc3,
       .byte 5,>__and,<__and          ; 0xc4,
       .byte 5,>__bit,<__bit          ; 0xc5,
       .byte 5,>__lda,<__lda          ; 0xc6,
       .byte 5,>__sta,<__sta          ; 0xc7,
       .byte 5,>__eor,<__eor          ; 0xc8,
       .byte 5,>__adc,<__adc          ; 0xc9,
       .byte 5,>__ora,<__ora          ; 0xca,
       .byte 5,>__add,<__add          ; 0xcb,
       .byte 5,>__jmp,<__jmp          ; 0xcc,
       .byte 5,>__jsr,<__jsr          ; 0xcd,
       .byte 5,>__ldx,<__ldx          ; 0xce,
       .byte 5,>__stx,<__stx          ; 0xcf,
       .byte 24,>__sub,<__sub         ; 0xd0,
       .byte 24,>__cmp,<__cmp         ; 0xd1,
       .byte 24,>__sbc,<__sbc         ; 0xd2,
       .byte 24,>__cpx,<__cpx         ; 0xd3,
       .byte 24,>__and,<__and         ; 0xd4,
       .byte 24,>__bit,<__bit         ; 0xd5,
       .byte 24,>__lda,<__lda         ; 0xd6,
       .byte 24,>__sta,<__sta         ; 0xd7,
       .byte 24,>__eor,<__eor         ; 0xd8,
       .byte 24,>__adc,<__adc         ; 0xd9,
       .byte 24,>__ora,<__ora         ; 0xda,
       .byte 24,>__add,<__add         ; 0xdb,
       .byte 24,>__jmp,<__jmp         ; 0xdc,
       .byte 24,>__jsr,<__jsr         ; 0xdd,
       .byte 24,>__ldx,<__ldx         ; 0xde,
       .byte 24,>__stx,<__stx         ; 0xdf,
       .byte 12,>__sub,<__sub         ; 0xe0,
       .byte 12,>__cmp,<__cmp         ; 0xe1,
       .byte 12,>__sbc,<__sbc         ; 0xe2,
       .byte 12,>__cpx,<__cpx         ; 0xe3,
       .byte 12,>__and,<__and         ; 0xe4,
       .byte 12,>__bit,<__bit         ; 0xe5,
       .byte 12,>__lda,<__lda         ; 0xe6,
       .byte 12,>__sta,<__sta         ; 0xe7,
       .byte 12,>__eor,<__eor         ; 0xe8,
       .byte 12,>__adc,<__adc         ; 0xe9,
       .byte 12,>__ora,<__ora         ; 0xea,
       .byte 12,>__add,<__add         ; 0xeb,
       .byte 12,>__jmp,<__jmp         ; 0xec,
       .byte 12,>__jsr,<__jsr         ; 0xed,
       .byte 12,>__ldx,<__ldx         ; 0xee,
       .byte 12,>__stx,<__stx         ; 0xef,
       .byte 13,>__sub,<__sub         ; 0xf0,
       .byte 13,>__cmp,<__cmp         ; 0xf1,
       .byte 13,>__sbc,<__sbc         ; 0xf2,
       .byte 13,>__cpx,<__cpx         ; 0xf3,
       .byte 13,>__and,<__and         ; 0xf4,
       .byte 13,>__bit,<__bit         ; 0xf5,
       .byte 13,>__lda,<__lda         ; 0xf6,
       .byte 13,>__sta,<__sta         ; 0xf7,
       .byte 13,>__eor,<__eor         ; 0xf8,
       .byte 13,>__adc,<__adc         ; 0xf9,
       .byte 13,>__ora,<__ora         ; 0xfa,
       .byte 13,>__add,<__add         ; 0xfb,
       .byte 13,>__jmp,<__jmp         ; 0xfc,
       .byte 13,>__jsr,<__jsr         ; 0xfd,
       .byte 13,>__ldx,<__ldx         ; 0xfe,
       .byte 13,>__stx,<__stx         ; 0xff,

; mnemonic names

__:      .byte 0
__bad:   .asciz '?'
__adc:   .asciz 'adc'
__add:   .asciz 'add'
__ais:   .asciz 'ais'
__aix:   .asciz 'aix'
__and:   .asciz 'and'
__asr:   .asciz 'asr'
__asra:  .asciz 'asra'
__asrx:  .asciz 'asrx'
__bclr:  .asciz 'bclr'
__beq:   .asciz 'beq'
__bge:   .asciz 'bge'
__bgt:   .asciz 'bgt'
__bhcc:  .asciz 'bhcc'
__bhcs:  .asciz 'bhcs'
__bhi:   .asciz 'bhi'
__bhs:   .asciz 'bhs'
__bih:   .asciz 'bih'
__bil:   .asciz 'bil'
__bit:   .asciz 'bit'
__ble:   .asciz 'ble'
__blo:   .asciz 'blo'
__bls:   .asciz 'bls'
__blt:   .asciz 'blt'
__bmc:   .asciz 'bmc'
__bmi:   .asciz 'bmi'
__bms:   .asciz 'bms'
__bne:   .asciz 'bne'
__bpl:   .asciz 'bpl'
__bra:   .asciz 'bra'
__brclr: .asciz 'brclr'
__brn:   .asciz 'brn'
__brset: .asciz 'brset'
__bset:  .asciz 'bset'
__bsr:   .asciz 'bsr'
__cbeq:  .asciz 'cbeq'
__cbeqa: .asciz 'cbeqa'
__cbeqx: .asciz 'cbeqx'
__clc:   .asciz 'clc'
__cli:   .asciz 'cli'
__clr:   .asciz 'clr'
__clra:  .asciz 'clra'
__clrh:  .asciz 'clrh'
__clrx:  .asciz 'clrx'
__cmp:   .asciz 'cmp'
__com:   .asciz 'com'
__coma:  .asciz 'coma'
__comx:  .asciz 'comx'
__cphx:  .asciz 'cphx'
__cpx:   .asciz 'cpx'
__daa:   .asciz 'daa'
__dbnz:  .asciz 'dbnz'
__dbnza: .asciz 'dbnza'
__dbnzx: .asciz 'dbnzx'
__dec:   .asciz 'dec'
__deca:  .asciz 'deca'
__decx:  .asciz 'decx'
__div:   .asciz 'div'
__eor:   .asciz 'eor'
__inc:   .asciz 'inc'
__inca:  .asciz 'inca'
__incx:  .asciz 'incx'
__jmp:   .asciz 'jmp'
__jsr:   .asciz 'jsr'
__lda:   .asciz 'lda'
__ldhx:  .asciz 'ldhx'
__ldx:   .asciz 'ldx'
__lsl:   .asciz 'lsl'
__lsla:  .asciz 'lsla'
__lslx:  .asciz 'lslx'
__lsr:   .asciz 'lsr'
__lsra:  .asciz 'lsra'
__lsrx:  .asciz 'lsrx'
__mov:   .asciz 'mov'
__mul:   .asciz 'mul'
__neg:   .asciz 'neg'
__nega:  .asciz 'nega'
__negx:  .asciz 'negx'
__nop:   .asciz 'nop'
__nsa:   .asciz 'nsa'
__ora:   .asciz 'ora'
__psha:  .asciz 'psha'
__pshh:  .asciz 'pshh'
__pshx:  .asciz 'pshx'
__pula:  .asciz 'pula'
__pulh:  .asciz 'pulh'
__pulx:  .asciz 'pulx'
__rol:   .asciz 'rol'
__rola:  .asciz 'rola'
__rolx:  .asciz 'rolx'
__ror:   .asciz 'ror'
__rora:  .asciz 'rora'
__rorx:  .asciz 'rorx'
__rsp:   .asciz 'rsp'
__rti:   .asciz 'rti'
__rts:   .asciz 'rts'
__sbc:   .asciz 'sbc'
__sec:   .asciz 'sec'
__sei:   .asciz 'sei'
__sta:   .asciz 'sta'
__sthx:  .asciz 'sthx'
__stop:  .asciz 'stop'
__stx:   .asciz 'stx'
__sub:   .asciz 'sub'
__swi:   .asciz 'swi'
__tap:   .asciz 'tap'
__tax:   .asciz 'tax'
__tpa:   .asciz 'tpa'
__tst:   .asciz 'tst'
__tsta:  .asciz 'tsta'
__tstx:  .asciz 'tstx'
__tsx:   .asciz 'tsx'
__txa:   .asciz 'txa'
__txs:   .asciz 'txs'
__wait:  .asciz 'wait'
.endif

;--------------------------------------------------------------------*/
;                       Utility routines                             */
;--------------------------------------------------------------------*/

;------------------------------------------------;
; rdxword - read the next word from command line ;
; o: H:X - value                                 ;
;    cy =1 syntax error                          ;
;------------------------------------------------;

rdxword:  clr    m_op5
          clr    m_op5+1

          jsr    spaces

rdxw_0:   ; find first digit
          jsr    lgetch
          cmp    #'\r'
          beq    rdxw_e     ; end of line

          jsr    hexdig
          bcs    rdxw_e     ; no hexa digit


; take digit
rdxw_3:   lsl    m_op5+1     ; shift value left by 4 bit
          rol    m_op5
          lsl    m_op5+1
          rol    m_op5
          lsl    m_op5+1
          rol    m_op5
          lsl    m_op5+1
          rol    m_op5

          ora    m_op5+1     ; or in digit
          sta    m_op5+1

          jsr   lgetch       ; get next digit
          cmp   #'\r'
          beq   rdxw_40
          cmp   #' '
          beq   rdxw_40
          cmp   #'\t'
          beq   rdxw_40      ; digit terminator

          jsr    hexdig
          bcc    rdxw_3      ; next hex digit

rdxw_e:   sec                ; bad syntax
          rts

rdxw_40:  clc                ; number read
          ldhx   m_op5
          rts




;-------------------;
; read a line       ;
;-------------------;
getln::  psha                ; room for input char on stack

         clr    m_pos        ; start from the beginning of line

         jsr    wrtsp         ; backspace safe space

getln1:  jsr    rdchr
         sta    1,sp         ; store char

.if IO_channel-BDM_channel
.else
	jsr    wrtchr
.endif


         cmp    #'o'         ; "over step" shortcut
         beq    getln10

         cmp    #'n'         ; "next step"  shortcut
         bne    getln2

getln10: tst    m_pos
         bne    getln11

         sta    m_line       ; 'n' or 'o' shortcut = 'n\r'
         inc    m_pos
         lda    #'\r'
         sta    1,sp


getln11: cmp    #'\b'
         bne    getln2

         jsr    wrtsp        ; restore cursor position due to BS on line

         lda    m_pos        ; Back Space
         beq    getln1

         dec    m_pos        ; real char cleared

         lda     #'\b'       ; step back to empty char
         jsr    wrtchr
         bra    getln1

getln2:

; CR can always fit in buffer
         lda    1,sp
         cmp    #'\r'
         beq    getln3

; Is there room for it ?
         lda    m_pos
         cmp    #m_xpos-1
         bhs    getln1       ; no room

; put into line buffer
getln3:  clrh
         ldx    m_pos
         lda    1,sp
         sta    m_line,x

         inc    m_pos

         cmp    #'\r'
         bne    getln1

         lda    #'\n'
         jsr    wrtchr

         clr    m_pos
         pula
         rts


;------------------------------;
; spaces - read over spaces    ;
;------------------------------;
spaces:  clrh
         ldx    m_pos
         lda    m_line,x
         cmp    #' '
         beq    spacn
         cmp    #'\t'
         beq    spacn
         rts
spacn:   inc    m_pos
         bra    spaces

;-----------------------------------;
; lgetch - read next char from line ;
; O: A  =char                       ;
;-----------------------------------;
lgetch:  clrh
         ldx    m_pos
         lda    m_line,x
         cmp    #'\r'
         beq    lgetche

         inc    m_pos

lgetche: rts


;----------------------------------------;
; lpeekch - peek next char, but leave it ;
; O: A  =char                            ;
;----------------------------------------;
lpeekch: clrh
         ldx    m_pos
         lda    m_line,x
         rts


;----------------------------------------;
;  rdchr - karakter olvass vgrehajtsa ;
;----------------------------------------;
; O: A  - a karakter                     ;
;    CY - ESC or CTRL-C                  ;
;----------------------------------------;
;     _getch() get a character, echo, returns in a-reg
;             returns
;
.if SERIAL_FROM_MDEB08

.if IO_channel-SCI_channel
.else
rdchr:
GetByte:   sta     COPCTL
           brclr   #SCRF_no,*SCS1,GetByte    ;SCRF = 1 ?
           lda     SCDR                      ;OK, get
           cmp     #0x1b                     ;ESC
           beq     GetByte_c
           cmp     #0x03                     ;CTRL+C
           bne     GetByte_e
GetByte_c: sec
           rts
GetByte_e: clc
           rts
.endif


.if IO_channel-BDM_channel
.else
rdchr::    pshx
           tpa                     ; disable IT-s
           psha
           sei

; Processor firmware GetByte (0xfeb0) routine

GetByte:   sta   0xFFFF             ; watchdog
           brset #0,*0x00,GetByte     ; wait start bit edge 1->0

           bsr   GetBit
           bcs   GetByte           ; it was shorter then a start bit

           lda   #0x80              ; set "end of byte" marker
GetNextBit:bsr   GetBit            ; get next bit
           rora                    ; put it inbyte
           ;nop
           bcc   GetNextBit
           bsr   GetBit            ; Check stop bit
            ; bcc   GetBreak       ; signals a break !

           tax                     ; save result
           pula                    ; restore IT mask
           tap
           txa                     ; result in A
           pulx                    ; restore X
           rts

GetBit:    pshx
           psha
           lda   #11
           ldx   #23               ; set sample count
GetBit_w:  brclr #0,*0x00,GetBit_e    ; accumulate samples in AC
GetBit_e:  sbc   #0x00
           nop
           dbnzx GetBit_w
           nop
           rola                    ; get result of accumulated samples in CY
           pula
           pulx
           rts
.endif

.else

; application exports DebGetByte()
           .globl _DebGetByte
rdchr:     pshx
           pshh
           jsr _DebGetByte
           pulh
           pulx
           rts
.endif


;----------------------------------;
; wrtword - write out a word value ;
; i: H:X = word value              ;
;----------------------------------;
wrtword::pshx
         pshh
         pula
         bsr  wrtbyte
         pula

;---------------------------------;
; wrtbyte - write out a byte value;
; i: A  = byte value              ;
;---------------------------------;
wrtbyte::
        psha

        nsa
        bsr  wrthdig    ; upper nibble

        pula            ; lower nibble

wrthdig:and   #0xf       ; take lower 4 bits
        cmp   #10
        bhs   wrthdia   ; A..F

        add  #'0'
        bra  wrtchr

wrthdia:add  #'A'-10
        bra   wrtchr


wrtcomma: lda  #','
          bra    wrtchr

wrtsp:    lda    #' '

;----------------------------;
;  wrtchr - write out a char ;
;----------------------------;
; I: A  - a karakter         ;
;----------------------------;
.if SERIAL_FROM_MDEB08

.if IO_channel-SCI_channel
.else
wrtchr::  sta     COPCTL
          brclr   #SCTE_no,*SCS1,wrtchr
          sta     SCDR            ; also SCTE is cleared here
          rts
.endif

.if IO_channel-BDM_channel
.else
wrtchr::   pshx                    ; save X
           psha                    ; save char

           tpa                     ; disable IT-s
           psha
           sei

; Modified processor firmware PutByte (0xfec3) routine

           bclr  #0,*0x00
           lda   #0x0A               ; bit count: start+8data+stop

PutByte_f: sta   0xFFFF              ; may be a long time, so fresh watchdog
           brclr #0,*0x00,PutByte_f    ; wait free line
           ldx   #169               ; make gap on line
PutByte_w1:dbnzx PutByte_w1
           sec                      ; set stop bit
           bra   PutByte_y

PutByte_n: ror   2,SP               ; get next bit, and put in stop bit, too
           bcc   PutByte_y
           bclr  #0,*0x04            ; 1 bit DDRA    = HIGH Z
           bra   PutByte_b

PutByte_y: bset  #0,*0x04           ; 0 bit  DDRA   = LOW
           bra   PutByte_b
PutByte_b: ldx   #79                ; wait out bit time
PutByte_w2:dbnzx PutByte_w2
           dbnza PutByte_n          ; check cycle end

           pula                     ; restore IT  mask
           tap

           pula
           pulx
           rts
.endif
.else
; application exports DebPutByte()
	   .globl _DebPutByte
wrtchr::   pshx
	   pshh
	   jsr _DebPutByte
	   pulh
	   pulx
	   rts
.endif

;---------------------------------------;
; wrttxt - write out crlf               ;
;---------------------------------------;
wrtcrlf: ldhx  #t_crlf

;---------------------------------------;
; wrttxt - write out 0 termnated string ;
; i: H:X - points to the string         ;
;---------------------------------------;
wrttxt:: lda   ,x
         beq   wrttxt_e

         pshx
         pshh
         bsr   wrtchr
         pulh
         pulx

         aix   #+1
         bra wrttxt

wrttxt_e: rts

.if FULL_SIZE
;---------------------------------------------;
; findreg - find the address of a register    ;
; i: register name on input line, e.g.  .ccr  ;
; o: m_beg  - memory address                  ;
;    cy - =1 not found                        ;
;---------------------------------------------;
findreg: jsr  lpeekch
         cmp  #'.'
         beq  fr1
fr0:     sec               ; not found
         rts

fr1:     ; read over dot
         jsr   lgetch

         ; find name

         ; from the start of register table
         ldhx #regtab
         sthx m_beg

fr2:     ; is it the end of table ?
         ldhx m_beg
         lda  2,x
         beq  fr0      ; end of table

         ldhx m_beg    ; get the name address of reg from the table */
         lda  2,x
         sta  m_op5
         lda  3,x
         sta  m_op5+1

         ; clear relative index
         clr  m_inx

fr3:     ; take the char of table name
         ldhx m_op5
         lda  ,x
         beq  fr4        ; name found, check if name in line ended

         ; take the char of input line
         clrh
         lda  m_pos
         add  m_inx
         tax
         lda  m_line,x

         jsr  to_upper     ; make it uppercase

         ldhx m_op5        ; compare it to register tab char
         cmp  ,x
         bne  fr_n         ; try next name

         ; step to the next char
         ldhx m_op5
         aix  #+1
         sthx m_op5

         inc  m_inx
         bra   fr3

; check if name is also terminated on input line
fr4:     clrh
         lda  m_pos
         add  m_inx
         tax
         lda  m_line,x

         cmp  #'\r'
         beq  fr5
         cmp  #'\t'
         beq  fr5
         cmp  #' '
         beq  fr5

; step to the next table item
fr_n:    ldhx  m_beg
         aix   #+4
         sthx  m_beg
         jmp   fr2


; name found, take the memory address
fr5:     lda   m_pos     ; step it over on input
         add   m_inx
         sta   m_pos

         ldhx  m_beg     ; get the register address
         lda   ,x
         ldx   1,x
         psha
         pulh
         sthx  *m_beg
         clc
         rts
.endif

;--------------------------------------------------------------;
; to_upper - make A uppercase
;--------------------------------------------------------------;

to_upper: cmp  #'a'         ; make it uppercase
          blo  to_upp1      ; lower
          cmp  #'z'
          bhi  to_upp1      ; higher

          sub  #0x20        ; offset it
to_upp1:  rts


.if FULL_SIZE

;--------------------------------------------------------------;
; regtab - processzor regiszterek nevnek s cmnek tblzata ;
;--------------------------------------------------------------;

regtab:
 .dw 0x00,__PTA
 .dw 0x01,__PTB
 .dw 0x02,__PTC
 .dw 0x03,__PTD
 .dw 0x04,__DDRA
 .dw 0x05,__DDRB
 .dw 0x06,__DDRC
 .dw 0x07,__DDRD
 .dw 0x08,__PTE
 .dw 0x09,__PTF
 .dw 0x0a,__PTG
 .dw 0x0b,__PTH
 .dw 0x0c,__DDRE
 .dw 0x0d,__DDRF
 .dw 0x0e,__DDRG
 .dw 0x0f,__DDRH
 .dw 0x10,__SPCR
 .dw 0x11,__SPSCR
 .dw 0x12,__SPDR
 .dw 0x13,__SCC1
 .dw 0x14,__SCC2
 .dw 0x15,__SCC3
 .dw 0x16,__SCS1
 .dw 0x17,__SCS2
 .dw 0x18,__SCDR
 .dw 0x19,__SCBR
 .dw 0x1a,__ISCR
 .dw 0x1b,__KBSCR
 .dw 0x1c,__PCTL
 .dw 0x1d,__PWBC
 .dw 0x1e,__PPG
 .dw 0x20,__TASC
 .dw 0x21,__KBICR
 .dw 0x22,__TACNTH
 .dw 0x23,__TACNTL
 .dw 0x24,__TAMODH
 .dw 0x25,__TAMODL
 .dw 0x26,__TASC0
 .dw 0x27,__TACH0H
 .dw 0x28,__TACH0L
 .dw 0x29,__TASC1
 .dw 0x2a,__TACH1H
 .dw 0x2b,__TACH1L
 .dw 0x2c,__TASC2
 .dw 0x2d,__TACH2H
 .dw 0x2e,__TACH2L
 .dw 0x2f,__TASC3
 .dw 0x30,__TACH3H
 .dw 0x31,__TACH3L
 .dw 0x38,__ADSCR
 .dw 0x39,__ADR
 .dw 0x3a,__ADCLK
 .dw 0x40,__TBSC
 .dw 0x41,__TBCNTH
 .dw 0x42,__TBCNTL
 .dw 0x43,__TBMODH
 .dw 0x44,__TBMODL
 .dw 0x45,__TBSC0
 .dw 0x46,__TBCH0H
 .dw 0x47,__TBCH0L
 .dw 0x48,__TBSC1
 .dw 0x49,__TBCH1H
 .dw 0x4a,__TBCH1L
 .dw 0x4b,__PSC
 .dw 0x4c,__PCNTH
 .dw 0x4d,__PCNTL
 .dw 0x4e,__PMODH
 .dw 0x4f,__PMODL
 .dw 0xfe00,__SBSR
 .dw 0xfe01,__SRSR
 .dw 0xfe03,__SBFCR
 .dw 0xfe0c,__BRKH
 .dw 0xfe0d,__BRKL
 .dw 0xfe0e,__BRKSCR
 .dw 0xfe0f,__LVISR
 .dw 0xfe1c,__EENVR
 .dw 0xfe1d,__EECR
 .dw 0xfe1f,__EEACR
 .dw 0xffff,__COP

 .dw s_CCR,__CCR
 .dw s_A,__A
 .dw s_X,__X
 .dw s_H,__H
 .dw s_PCH,__PCH
 .dw s_PCL,__PCL

 .dw  0x0000,0x0000      ; terminator record



; register names

__CCR:   .asciz 'CCR'
__A:     .asciz 'A'
__X:     .asciz 'X'
__H:     .asciz 'H'
__PCL:   .asciz 'PCL'
__PCH:   .asciz 'PCH'

__PTA:   .asciz 'PTA'
__PTB:   .asciz 'PTB'
__PTC:   .asciz 'PTC'
__PTD:   .asciz 'PTD'
__DDRA:  .asciz 'DDRA'
__DDRB:  .asciz 'DDRB'
__DDRC:  .asciz 'DDRC'
__DDRD:  .asciz 'DDRD'
__PTE:   .asciz 'PTE'
__PTF:   .asciz 'PTF'
__PTG:   .asciz 'PTG'
__PTH:   .asciz 'PTH'
__DDRE:  .asciz 'DDRE'
__DDRF:  .asciz 'DDRF'
__DDRG:  .asciz 'DDRG'
__DDRH:  .asciz 'DDRH'
__SPCR:  .asciz 'SPCR'
__SPSCR: .asciz 'SPSCR'
__SPDR:  .asciz 'SPDR'
__SCC1:  .asciz 'SCC1'
__SCC2:  .asciz 'SCC2'
__SCC3:  .asciz 'SCC3'
__SCS1:  .asciz 'SCS1'
__SCS2:  .asciz 'SCS2'
__SCDR:  .asciz 'SCDR'
__SCBR:  .asciz 'SCBR'
__ISCR:  .asciz 'ISCR'
__KBSCR: .asciz 'KBSCR'
__PCTL:  .asciz 'PCTL'
__PWBC:  .asciz 'PWBC'
__PPG:   .asciz 'PPG'
__TASC:  .asciz 'TASC'
__KBICR: .asciz 'KBICR'
__TACNTH:.asciz 'TACNTH'
__TACNTL:.asciz 'TACNTL'
__TAMODH:.asciz 'TAMODH'
__TAMODL:.asciz 'TAMODL'
__TASC0: .asciz 'TASC0'
__TACH0H:.asciz 'TACH0H'
__TACH0L:.asciz 'TACH0L'
__TASC1: .asciz 'TASC1'
__TACH1H:.asciz 'TACH1H'
__TACH1L:.asciz 'TACH1L'
__TASC2: .asciz 'TASC2'
__TACH2H:.asciz 'TACH2H'
__TACH2L:.asciz 'TACH2L'
__TASC3: .asciz 'TASC3'
__TACH3H:.asciz 'TACH3H'
__TACH3L:.asciz 'TACH3L'
__ADSCR: .asciz 'ADSCR'
__ADR:   .asciz 'ADR'
__ADCLK: .asciz 'ADCLK'
__TBSC:  .asciz 'TBSC'
__TBCNTH:.asciz 'TBCNTH'
__TBCNTL:.asciz 'TBCNTL'
__TBMODH:.asciz 'TBMODH'
__TBMODL:.asciz 'TBMODL'
__TBSC0: .asciz 'TBSC0'
__TBCH0H:.asciz 'TBCH0H'
__TBCH0L:.asciz 'TBCH0L'
__TBSC1: .asciz 'TBSC1'
__TBCH1H:.asciz 'TBCH1H'
__TBCH1L:.asciz 'TBCH1L'
__PSC:   .asciz 'PSC'
__PCNTH: .asciz 'PCNTH'
__PCNTL: .asciz 'PCNTL'
__PMODH: .asciz 'PMODH'
__PMODL: .asciz 'PMODL'
__SBSR:  .asciz 'SBSR'
__SRSR:  .asciz 'SRSR'
__SBFCR: .asciz 'SBFCR'
__BRKH:  .asciz 'BRKH'
__BRKL:  .asciz 'BRKL'
__BRKSCR:.asciz 'BRKSCR'
__LVISR: .asciz 'LVISR'
__EENVR: .asciz 'EENVR'
__EECR:  .asciz 'EECR'
__EEACR: .asciz 'EEACR'
__COP:   .asciz 'COP'

.endif

 .if FULL_SIZE
;---------------------------------------------;
; h   - help szveg                           ;
;---------------------------------------------;
comm_h:  ldhx  #helptxt
         jsr   wrttxt
         jmp   command

helptxt:
 .ascii '\r\nDeb08 - HC08 Debugger V1.04 (C) 2003. SILEX Co. (Sarosi)\r\n\r\n'

 .ascii 'Status   : .                     Help: h  or  ?\r\n'
 .ascii 'Compare  : c begaddr endaddr toaddr\r\n'
 .ascii 'Dump     : d [begaddr [endaddr]        |  d .reg\r\n'
 .ascii 'Enter    : e begaddr byte1 byte2 ...   |  e .reg byte\r\n'
 .ascii 'Fill     : f begaddr endaddr byte1 byte2 ...\r\n'
 .ascii 'Go       : g [=startaddr] [brkaddr]\r\n'
 .ascii 'Next/Over: n  or  o\r\n'
 .ascii 'Move     : m begaddr endaddr toaddr\r\n'
 .ascii 'Search   : s begaddr endaddr byte1 byte2 ...\r\n'
 .ascii 'Unass.   : u [begaddr]\r\n'
 .ascii 'Registers: A X H CCR PCH PCL\r\n'
 .ascii ' PTA PTB PTC PTD DDRA DDRB DDRC DDRD PTE PTF PTG PTH DDRE DDRF DDRG DDRH\r\n'
 .ascii ' SPCR SPSCR SPDR SCC1 SCC2 SCC3 SCS1 SCS2 SCDR SCBR ISCR KBSCR PCTL PWBC\r\n'
 .ascii ' PPG TASC KBICR TACNTH TACNTL TAMODH TAMODL TASC0 TACH0H TACH0L TASC1 TACH1H\r\n'
 .ascii ' TACH1L TASC2 TACH2H TACH2L TASC3 TACH3H TACH3L ADSCR ADR ADCLK TBSC TBCNTH\r\n'
 .ascii ' TBCNTL TBMODH TBMODL TBSC0 TBCH0H TBCH0L TBSC1 TBCH1H TBCH1L PSC PCNTH\r\n'
 .asciz ' PCNTL PMODH PMODL SBSR SRSR SBFCR BRKH BRKL BRKSCR LVISR EENVR EECR EEACR COP\r\n'
.endif

 .if STANDALONE
         .area _ABSOLUTE(ABS)
         .org 0xfffc
         .dw SWI_IT

         .org 0xfffe
         .dw start
 .else
         .area _ABSOLUTE(ABS)
         .org 0xfffc
         .dw SWI_IT
 .endif

;end of DEB08.A08
