// sender.c
//
// TV Remote controller project for Motorola68HC908QY4
// Based on SDCC M08 Port Compiler & Library
// By Sarosi, 2004.03.07, Free
//

#include "mc68hc908qy.h"
#include "sender.h"


// SCI output definitions

unsigned char  _near oBitType[oBits];  // contains the ticks for bits to shift out
unsigned char  * _near oNextBit;        // next bit type index
unsigned char  _near oBitCnt;         // bit count in oBitTicks
unsigned int   _near oBitActivTicks;  // contains the ticks for bits to shift out

unsigned char _near SciPort;

unsigned char _near SciStat;
unsigned int  _near SciTmr;

#define BIT_0 0
#define BIT_1 1
#define BIT_HEADER 2
#define BIT_START 3
#define BIT_STOP 4

// bit time table
const unsigned int oBitTicks[5] ={
  (1*T_TICK),  // BIT_0
  (2*T_TICK),  // BIT_1
  (3*T_TICK),  // BIT_HEADER
  (1*T_TICK),  // BIT_START
  (1*T_TICK)   // BIT_STOP
};


//-----------------------------------------------
//-----------------------------------------------
void SCIInit(void) {
   TMOD = TMRMODULO;
   TSC = 0; // run timer, no prescaling, no modulo int.
   //TXDPINPUE = 1; // enable pull-up
}


//-----------------------------------------------
//-----------------------------------------------
void oWrite(unsigned int oCode) {

   unsigned int w;
   unsigned char b;

   // precalculate the tick counts of the code

   // Header bit
   oBitType[0] = BIT_HEADER;

   // Start bit
   oBitType[1] = BIT_START;

   // 10 data bits
   oNextBit = & oBitType[11];
   for( b=0; b<10; b++ ){
      *oNextBit = (unsigned char) oCode & 1 ;
      oCode >>= 1;
      oNextBit++;
   }

   // Stop bit
   oBitType[12] = BIT_STOP;

   oNextBit = & oBitType[0];
   oBitCnt  = 13;

   #ifndef SCIINV
      TSC0 = 0x00; // reset timer logic so no false edge appears
   #else
      TSC0 = 0x10; // reset timer logic so no false edge appears
   #endif

   #ifndef SCIINV
      TSC0 = 0x18; // output compare, falling edge
   #else
      TSC0 = 0x1C; // output compare, rising edge
   #endif
   w = oBitTicks[*oNextBit];
   SciTmr = TCNT;
   SciTmr += w;
   SciTmr &= TMRMODULO;
   TCH0 = SciTmr;
   oNextBit++;
   CH0F = 0; // clearing timer flag
   CH0IE = 1; // enable tmr. channel interrupts
}


//-----------------------------------------------
// Timer_IT - IT vector still must be planted anyway
//-----------------------------------------------
void Timer_IT(void) interrupt 0xfff4 {
   unsigned int w;

   w = oBitTicks[*oNextBit];

   if (oBitCnt > 1) {
      SciTmr += w;
      SciTmr &= TMRMODULO;
      TCH0 = SciTmr;
      oNextBit++;
      oBitCnt--;

/*
      #ifndef SCIINV
         TSC0 = 0x58 | (SciBuff & 0x01?0x04:0); // output compare, schedule clear output (start bit)
      #else
         TSC0 = 0x58 | (!(SciBuff & 0x01)?0x04:0); // output compare, schedule clear output (start bit)
      #endif
*/
      #ifndef SCIINV
         TSC0 = 0x18; // output compare, falling edge
      #else
         TSC0 = 0x1C; // output compare, rising edge
      #endif

   } else if (oBitCnt == 1){ // stop bit reached

      #ifndef SCIINV
            TSC0 = 0x58 | 0x04; // output compare, schedule set output (stop bit)
      #else
            TSC0 = 0x58; // output compare, schedule set output (stop bit)
      #endif

      SciTmr += w;
      SciTmr &= TMRMODULO;
      TCH0 = SciTmr;
      oNextBit++;
      oBitCnt--;
   } else {
      TSC0 = 0x00; // port control, set output & disable further interrupts
   }
   CH0F = 0; // clearing timer flag

   // plant IT vector
   _asm
   .area _ABSOLUTE(ABS)
   .org 0xfff4
   .dw _Timer_IT
   .area _CODE
   _endasm;
}

// End of sender

