// sender.c
//
// TV Remote controller project for Motorola68HC908QY4
// Based on SDCC M08 Port Compiler & Library
// By Sarosi, 2004.03.07, Free
//

#ifndef SENDER_H
#define SENDER_H

#include "mc68hc908qy.h"
#include "rc.h"

/*
Internal Oscillator
Usage
This application uses the internal oscillator of the QT/QY M68HC08 MCU. The
internal oscillator is specified to run at 12.8 MHz, +25%. The bus clock is then
3.2 MHz, +25%.
The +25% variation can be reduced to +5% by trimming the oscillator. The
MCU has a factory pre-programmed trim value at address 0xFFC0 that has
been measured at the time of testing. However, there is no guarantee that this
value will work with SCI communication.
*/


#define BUS_CLOCK_HZ 3000000 /* reqd' bus clock in Hz */
#define T_RATE 10000L
#define TMRMODULO 0x3fff // specify the modulo (mask) in which 'free' running timer operates
#define T_TICK (BUS_CLOCK_HZ / T_RATE)



/*------SCI definitions registers-------------------*/
/* common softSCI section */
#define SCISINGLEWIRE // define only if RXD & TXD pins are shared (ie. single wire)
#define SCIINV // define this one, if SCI needs to be inverted (ie. non-standard interface)

/*### TXD pin section */

#define TXDPIN PTA0
#define TXDPINDDR DDRA_BIT0
#define TXDPINPUE PTAPUE_BIT0


/*### RXD pin section */
#define SCIRXDPINISTIMERPIN // defined if RXD pin can use hw input capture feature
#define RXDPIN PTA0
#define RXDPINDDR DDRA_BIT0
#define RXDPINPORT PTA
#define RXDPINMASK 0x01


#define DDRIN 0
#define DDROUT 1

#ifndef SCIINV
#define TXDPINSET() TXDPIN=1
#define TXDPINCLR() TXDPIN=0
#else
#define TXDPINSET() TXDPIN=0
#define TXDPINCLR() TXDPIN=1
#endif



// SCI output definitions

#define oBits 13            // H+S+10+E
extern unsigned int   _near oBitTicks[oBits];  // contains the ticks for bits to shift out


extern unsigned char  _near oBitCnt;         // bit count in oBitTicks
extern unsigned int * _near oNextTick;       // contains the ticks for bits to shift out
extern unsigned int   _near oBitActivTicks;  // contains the ticks for bits to shift out

extern unsigned char _near SciPort;

extern unsigned char _near SciStat;
extern unsigned int  _near SciTmr;


void SCIInit(void);
void oWrite(unsigned int oCode);
void Timer_IT(void) interrupt 0xfff4;

#endif
// End of sender.c

