// TV Remote controller project for Motorola68HC908QY4
// Based on SDCC M08 Port Compiler & Library
// By Sarosi, 2004.03.07, Free
//

#include "serial.h"

// Timing constans calculated as: 
// 9600,8,N,1 => 104 us/bit => 363 cycle/bit

#define CYCLES_PER_BIT (363)
#define READ_SAMPLES   ((CYCLES_PER_BIT-25)/11)
#define SAMPLES_TO_MATCH    (READ_SAMPLES/2)
#define BIT_LOOPS ((CYCLES_PER_BIT-18)/3)
#define INTERCHAR_LOOPS (BIT_LOOPS + (BIT_LOOPS/2))

const char hexDigits[17]="0123456789ABCDEF";

//-----------------------------------------------
// DebGetByte - reads a byte from serial line (PTA0, 9600,8N1)
//-----------------------------------------------
char DebGetByte(void){
_asm
            tpa                     // disable IT-s
            psha                    // save status
            sei
GetByte:            
            sta   0xFFFF            // watchdog
            brset #0,*0x00,GetByte  // wait start bit edge 1->0

            bsr   GetBit
            bcs   GetByte           // it was shorter then a start bit

            lda   #0x80             // set "end of byte" marker
GetNextBit: 
            bsr   GetBit            // get next bit
            rora                    // put it inbyte
            ;nop
            bcc   GetNextBit
            bsr   GetBit            // Check stop bit
            ; bcc   GetBreak        // signals a break !

            tax                     // save result
            pula                    // restore IT mask
            tap
            txa                     // result in A
            rts

GetBit:     
            pshx
            psha
            lda   #SAMPLES_TO_MATCH      // 15 count of samles MUST MATCH FO A BIT
            ldx   #READ_SAMPLES          // 30 set bit sample count
GetBit_w:   
            brclr #0,*0x00,GetBit_e // accumulate samples in AC
GetBit_e:   
            sbc   #0x00
            nop
            dbnzx GetBit_w
            nop
            rola                    // get result of accumulated samples in CY
            pula
            pulx
            rts
_endasm;
return 0;
}


//-----------------------------------------------
// PutByte - write a byte to serial line (PTA0, 9600,8N1)
//-----------------------------------------------
void DebPutByte(unsigned char byte){
 byte+0;
_asm
           psha                // save char

           tpa                 // disable IT-s
           psha                // save status
           sei

// as processor firmware PutByte (0xfec3?) routine

           bclr  #0,*0x00
           lda   #0x0A          // bit count: start+8data+stop

PutB_f:    sta   0xFFFF         // may be a long time, so fresh watchdog
           brclr #0,*0x00,PutB_f  // wait free line
           ldx   #INTERCHAR_LOOPS      // 180 make gap on line
PutB_w1:   dbnzx PutB_w1
           sec                  // set stop bit (carry is not changed during bit timing)
           bra   PutB_y     	// force start as 0

PutB_n:    
           ror   2,SP           // get next bit, and put in stop bit, too
           bcc   PutB_y
           bclr  #0,*0x04       // 1 bit DDRA    = HIGH Z
           bra   PutB_b

PutB_y:    
           bset  #0,*0x04       // 0 bit  DDRA   = LOW
           bra   PutB_b

PutB_b:    
           ldx   #BIT_LOOPS    // 115 wait out bit time
PutB_w2:   
           dbnzx PutB_w2

           dbnza PutB_n         // check cycle end

           pula                 // restore IT  mask
           tap

           pula                 // restore stack
_endasm;
}


//-----------------------------------------------
// DebPuts - write a string to serial line
//-----------------------------------------------
void DebPuts(char *str){
   char c;
   while( c=*str ){
      DebPutByte( c );
      str++;
   }
}


void DebPutB(unsigned char uc){
  DebPutByte( hexDigits[ uc>>4 ] );
  DebPutByte( hexDigits[ uc&0x0f ] );
}


void DebPutW(unsigned int uw){
  DebPutB( uw>>8 );
  DebPutB( uw&0xff );
}

#if 0
void DebPutDW(unsigned pong ul){
  DebPutW( ul>>16 );
  DebPutW( ul&0xffff );
}
#endif
