// sender.c
//
// TV Remote controller project for Motorola68HC908QY4
// Based on SDCC M08 Port Compiler & Library
// By Sarosi, 2004.03.07, Free
//
#include "rc.h"
#include "delay.h"
#include "serial.h"

#include "tda1250.h"

/*


Coding


  +-+                  +-+      +-+      +-+            +-+                  +-+
  |t|      3*T         |t| T    |t| T    |t|   2*T      |t|      3*T         |t|  3*T 
  | |                  | |      | |      | |            | |                  | |
--+ +------------------+ +------+ +------+ +------------+ +------------------+ +------
  |        PRE         |   STA  |   0    |       1      |        STO         |    CLO
                                |<-  4+6 data bits    ->|

t = 10us
T = 100us
PRE = preliminary bit (300us)
STA = startbit (100us)
0 = "0" bit (100us)
1 = "1" bit (200us)
STO = stopbit (300us)
CLO = closing bit (300us)

*/

#define T_10us 10  // equals 100 us 

#define PRE (3*T_10us)
#define STA (1*T_10us)
#define BIT0 (1*T_10us)
#define BIT1 (2*T_10us)
#define STO (3*T_10us)
#define CLO (3*T_10us)


//-----------------------------------------------
// TDA1250_bit - sends a TDA1250 specific bit
//-----------------------------------------------
static void TDA1250_bit(unsigned char t_10us) {
   DebPutByte('1');
   PTA1 = 0;
   Delay1us(10);
   DebPutByte('0');
   PTA1 = 1;
   Delay10us(t_10us); 
}

//-----------------------------------------------
//TDA1250_code - sends a TDA1250 specific 10-bit code
//-----------------------------------------------
void TDA1250_code(unsigned int tenbits) {

#if 0 
   tenbits+0; 
   DDRA1 = 1;
   DDRA3 = 1;
   while(1){
      PTA1 = 0;
      PTA3 = 0;
      DelayMs(3000);
      PTA1= 1;
      PTA3 = 1;
      DelayMs(3000);
   }

#else

   w = tenbits;
   DDRA1 = 1;
   TDA1250_bit(PRE);
   TDA1250_bit(STA);

   // 10 data bits
   
   for( b=10; b; b--){
      if( W_BIT9 ){
         TDA1250_bit(BIT1);
      } else {
         TDA1250_bit(BIT0);
      }
      w <<= 1;
   }
   TDA1250_bit(STO);
   TDA1250_bit(CLO);
   DDRA1 = 0;
#endif

}

// End of sender


