// SDCC M08 Port Library & compiler Test
// By Sarosi, 2004, Free
//
#include "stdio.h"
#include "float.h"
#include "math.h"
#include "string.h"

#define TEST_CHAR
#define TEST_UCHAR
#define TEST_INT
#define TEST_UINT
//#define TEST_LONG
//#define TEST_ULONG
#define TEST_FLOAT
//#define TEST_BITS
//#define TEST_SWITCH


// Proba program
int errno;
char xb;
int xw;

// Call monitor program (go into debugger)
extern void monitor(void );

// deb_blink3 - blink on LED 3 times
extern void DebBlink3(void );

// deb_getb - read a byte from debug input (SCI)
extern unsigned char DebGetb( void );

// deb_blink - blink on LED count times
void DebBlink( unsigned char count){
  count+0;
  _asm
   .globl __deb_blink
   jmp __deb_blink
  _endasm;
}

// deb_putb - write a byte to debug output (SCI)
void DebPutb( unsigned char value){
  value+0;
  _asm
   .globl __deb_putb
   jmp __deb_putb
  _endasm;
}

void putchar(char ch){
  ch+0;
  _asm
   .globl __deb_putb
   jmp __deb_putb
  _endasm;
}

// deb_putbx - write a byte as 2 hex ASCII digits to debug output (SCI)
void DebPutbx( unsigned char value){
  value+0;
  _asm
   .globl __deb_putbx
   jmp __deb_putbx
  _endasm;
}

// deb_putwx - write a word as 4 hex ASCII digits to debug output (SCI)
void DebPutwx( unsigned int value){
  value+0;
  _asm
   .globl __deb_putwx
   pshx
   tax
   pulh
   jmp __deb_putwx
  _endasm;
}

// deb_putdx - write a double word as 8 hex ASCII digits to debug output (SCI)
void DebPutdx( unsigned long value){
  value+0;
  _asm
   tsx
   lda 2,x
   ldx 3,x
   psha
   pulh
   jsr __deb_putwx

   tsx
   lda 4,x
   ldx 5,x
   psha
   pulh
   jmp __deb_putwx

  _endasm;
}



// deb_puts - write a 0 terminated string to debug output (SCI)
void DebPuts( char *str){
  str+0;
  _asm
   .globl __deb_wrttxt
   pshx
   tax
   pulh
   jsr __deb_wrttxt
  _endasm;
}

// deb_gets - reads a 0 terminated string from debug input (SCI)
char *DebGets( char *str, char size){
  int backup;
  char *next;

  DebPuts(" ");    // Back Space safe !
  next = str;
  while(--size > 0) {
    switch (*next = DebGetb()) {
      case '\r': DebPuts("\n");
      case '\n': *next= 0;
                 return (str);

      case '\b': if(next > str){
                  DebPuts(" \b");
                  --next;
                  size += 2;
                 } else {
                  DebPuts(" ");
                  ++size;
                 }
                 break;

      case 0x1b: backup = next - str;
                 ++size;
                 DebPuts("\b \b");  // clear ESC
                 while(backup--) {
                   DebPuts("\b \b");
                   --next; ++size;
                 }
                 break;

        default: ++next;
      }
    }
  *next = 0;
  return (str);
}
  
void DebPutULong(char *prompt, unsigned long ul){
  DebPuts("{");
  DebPuts(prompt);
  DebPuts(":");
  DebPutdx( ul );
  DebPuts("}");
}

void DebPutFloat(char *prompt, float f){
  DebPutULong(prompt, __AS_ULONG(f));
}



float a,b,c;
//unsigned long
//char
//unsigned char
unsigned long
//unsigned int
i=0x01234567L,
j=1L,
k;

_near char c1;
_near char c2;
_near char c3;
char * _near pc1;
char * _near pc2;
char * _near pc3;

_near unsigned char uc1, uc2, uc3;
unsigned char * _near puc1, * _near puc2, * _near puc3;

int i1, i2, i3;
int *pi1, *pi2, *pi3;

unsigned int ui1, ui2, ui3;
unsigned int *pui1, *pui2, *pui3;

long l1,l2,l3;
long *pl1,*pl2,*pl3;

unsigned long ul1,ul2,ul3;
unsigned long *pul1,*pul2,*pul3;

//int  _near at(0x56) reg;
//char _near at(0x58) creg;




unsigned long l;

#define MSW(a) *((unsigned int *)(&a))
#define LSW(a) *((unsigned int *)(((char *)(&a))+2))

#define FtoL(a) *((unsigned long *)(&a))


#define CtoUI(c) (unsigned int)((unsigned char) (c))


//char buff[30];
float f1=HALF_PI,  f2=0.533, f3;


#ifdef TEST_UCHAR



void test_uchar_uc123(char op, char *result){
  char r[20];

  switch(op){
  case '+':  uc3 = uc1+uc2;  break;
  case '-':  uc3 = uc1-uc2;  break;
  case '*':  uc3 = uc1*uc2;  break;
  case '/':  uc3 = uc1/uc2;  break;
  case '%':  uc3 = uc1%uc2;  break;
  case '&':  uc3 = uc1&uc2;  break;
  case '|':  uc3 = uc1|uc2;  break;
  case '^':  uc3 = uc1^uc2;  break;
  case '<':  uc3 = uc1<<(unsigned char)uc2;  break;
  case '>':  uc3 = uc1>>(unsigned char)uc2;  break;
  case '~':  uc3 = ~uc1;  break;
  case '!':  uc3 = !uc1;  break;
  }
  sprintf(r,"%02X",CtoUI(uc3));
  printf("\nTest static UCHAR(%c): %02X %c %02X = %02X (%s) %s\n",
                 op,CtoUI(uc1),op,CtoUI(uc2),CtoUI(uc3),result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

}



void test_uchar_puc123(char op, char *result){
  char r[20];

  unsigned char  *apuc1, *apuc2, *apuc3;

  switch(op){
  case '+':  *puc3 = *puc1+ *puc2;  break;
  case '-':  *puc3 = *puc1- *puc2;  break;
  case '*':  *puc3 = *puc1* *puc2;  break;
  case '/':  *puc3 = *puc1/ *puc2;  break;
  case '%':  *puc3 = *puc1% *puc2;  break;
  case '&':  *puc3 = *puc1& *puc2;  break;
  case '|':  *puc3 = *puc1| *puc2;  break;
  case '^':  *puc3 = *puc1^ *puc2;  break;
  case '<':  *puc3 = *puc1<<(unsigned char)(*puc2);  break;
  case '>':  *puc3 = *puc1>>(unsigned char)(*puc2);  break;
  case '~':  *puc3 = ~*puc1;  break;
  case '!':  *puc3 = !*puc1;  break;
  }
  sprintf(r,"%02X",CtoUI(*puc3));
  printf("\nTest static * UCHAR(%c): %02X %c %02X = %02X (%s) %s\n",
                 op,CtoUI(*puc1),op,CtoUI(*puc2),CtoUI(*puc3),result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

apuc1=puc1;
apuc2=puc2;
apuc3=puc3;

switch(op){
case '+':  *apuc3 = *apuc1+ *apuc2;  break;
case '-':  *apuc3 = *apuc1- *apuc2;  break;
case '*':  *apuc3 = *apuc1* *apuc2;  break;
case '/':  *apuc3 = *apuc1/ *apuc2;  break;
case '%':  *apuc3 = *apuc1% *apuc2;  break;
case '&':  *apuc3 = *apuc1& *apuc2;  break;
case '|':  *apuc3 = *apuc1| *apuc2;  break;
case '^':  *apuc3 = *apuc1^ *apuc2;  break;
case '<':  *apuc3 = *apuc1<<(unsigned char)(*apuc2);  break;
case '>':  *apuc3 = *apuc1>>(unsigned char)(*apuc2);  break;
case '~':  *apuc3 = ~*apuc1;  break;
case '!':  *apuc3 = !*apuc1;  break;
}
sprintf(r,"%02X",CtoUI(*apuc3) );
printf("\nTest auto * UCHAR(%c): %02X %c %02X = %02X (%s) %s\n",
               op,CtoUI(*apuc1),op,CtoUI(*apuc2),CtoUI(*apuc3),result,
               strcmp(result, r) ? "ERROR!" : "OK"
       );
DebBlink(1);
if( strcmp(result, r)) monitor();

}


void test_uchar(char op, unsigned char pauc1, unsigned char pauc2, char *result){
  char r[20];
  unsigned char pauc3;

  switch(op){
  case '+':  pauc3 = pauc1+ pauc2;  break;
  case '-':  pauc3 = pauc1- pauc2;  break;
  case '*':  pauc3 = pauc1* pauc2;  break;
  case '/':  pauc3 = pauc1/ pauc2;  break;
  case '%':  pauc3 = pauc1% pauc2;  break;
  case '&':  pauc3 = pauc1& pauc2;  break;
  case '|':  pauc3 = pauc1| pauc2;  break;
  case '^':  pauc3 = pauc1^ pauc2;  break;
  case '<':  pauc3 = pauc1<<(unsigned char)pauc2;  break;
  case '>':  pauc3 = pauc1>>(unsigned char)pauc2;  break;
  case '~':  pauc3 = ~pauc1;  break;
  case '!':  pauc3 = !pauc1;  break;
  }
  sprintf(r,"%02X", CtoUI(pauc3) );
  printf("\nTest auto UCHAR(%c): %02X %c %02X = %02X (%s) %s\n",
                 op,CtoUI(pauc1),op,CtoUI(pauc2),CtoUI(pauc3),result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

  uc1= pauc1;
  uc2= pauc2;
  test_uchar_uc123(op, result);

  uc1= pauc1;
  uc2= pauc2;
  puc1= &uc1;
  puc2= &uc2;
  puc3= &uc3;
  test_uchar_puc123(op, result);

}
#endif



#ifdef TEST_CHAR



void test_char_c123(char op, char *result){
  char r[20];

  switch(op){
  case '+':  c3 = c1+c2;  break;
  case '-':  c3 = c1-c2;  break;
  case '*':  c3 = c1*c2;  break;
  case '/':  c3 = c1/c2;  break;
  case '%':  c3 = c1%c2;  break;
  case '&':  c3 = c1&c2;  break;
  case '|':  c3 = c1|c2;  break;
  case '^':  c3 = c1^c2;  break;
  case '<':  c3 = c1<<(unsigned char)c2;  break;
  case '>':  c3 = c1>>(unsigned char)c2;  break;
  case '~':  c3 = ~c1;  break;
  case '!':  c3 = !c1;  break;
  }
  sprintf(r,"%02X",CtoUI(c3));
  printf("\nTest static CHAR(%c): %02X %c %02X = %02X (%s) %s\n",
                 op,CtoUI(c1),op,CtoUI(c2),CtoUI(c3),result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

}



void test_char_pc123(char op, char *result){
  char r[20];

  char  *apc1, *apc2, *apc3;

  switch(op){
  case '+':  *pc3 = *pc1+ *pc2;  break;
  case '-':  *pc3 = *pc1- *pc2;  break;
  case '*':  *pc3 = *pc1* *pc2;  break;
  case '/':  *pc3 = *pc1/ *pc2;  break;
  case '%':  *pc3 = *pc1% *pc2;  break;
  case '&':  *pc3 = *pc1& *pc2;  break;
  case '|':  *pc3 = *pc1| *pc2;  break;
  case '^':  *pc3 = *pc1^ *pc2;  break;
  case '<':  *pc3 = *pc1<<(unsigned char)(*pc2);  break;
  case '>':  *pc3 = *pc1>>(unsigned char)(*pc2);  break;
  case '~':  *pc3 = ~*pc1;  break;
  case '!':  *pc3 = !*pc1;  break;
  }
  sprintf(r,"%02X",CtoUI(*pc3));
  printf("\nTest static * CHAR(%c): %02X %c %02X = %02X (%s) %s\n",
                 op,CtoUI(*pc1),op,CtoUI(*pc2),CtoUI(*pc3),result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

apc1=pc1;
apc2=pc2;
apc3=pc3;

switch(op){
case '+':  *apc3 = *apc1+ *apc2;  break;
case '-':  *apc3 = *apc1- *apc2;  break;
case '*':  *apc3 = *apc1* *apc2;  break;
case '/':  *apc3 = *apc1/ *apc2;  break;
case '%':  *apc3 = *apc1% *apc2;  break;
case '&':  *apc3 = *apc1& *apc2;  break;
case '|':  *apc3 = *apc1| *apc2;  break;
case '^':  *apc3 = *apc1^ *apc2;  break;
case '<':  *apc3 = *apc1<<(unsigned char)(*apc2);  break;
case '>':  *apc3 = *apc1>>(unsigned char)(*apc2);  break;
case '~':  *apc3 = ~*apc1;  break;
case '!':  *apc3 = !*apc1;  break;
}
sprintf(r,"%02X",CtoUI(*apc3) );
printf("\nTest auto * CHAR(%c): %02X %c %02X = %02X (%s) %s\n",
               op,CtoUI(*apc1),op,CtoUI(*apc2),CtoUI(*apc3),result,
               strcmp(result, r) ? "ERROR!" : "OK"
       );
DebBlink(1);
if( strcmp(result, r)) monitor();

}


void test_char(char op, char pac1, char pac2, char *result){
  char r[20];
  char pac3;

  switch(op){
  case '+':  pac3 = pac1+ pac2;  break;
  case '-':  pac3 = pac1- pac2;  break;
  case '*':  pac3 = pac1* pac2;  break;
  case '/':  pac3 = pac1/ pac2;  break;
  case '%':  pac3 = pac1% pac2;  break;
  case '&':  pac3 = pac1& pac2;  break;
  case '|':  pac3 = pac1| pac2;  break;
  case '^':  pac3 = pac1^ pac2;  break;
  case '<':  pac3 = pac1<<(unsigned char)pac2;  break;
  case '>':  pac3 = pac1>>(unsigned char)pac2;  break;
  case '~':  pac3 = ~pac1;  break;
  case '!':  pac3 = !pac1;  break;
  }
  sprintf(r,"%02X", CtoUI(pac3) );
  printf("\nTest auto CHAR(%c): %02X %c %02X = %02X (%s) %s\n",
                 op,CtoUI(pac1),op,CtoUI(pac2),CtoUI(pac3),result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

  c1= pac1;
  c2= pac2;
  test_char_c123(op, result);

  c1= pac1;
  c2= pac2;
  pc1= &c1;
  pc2= &c2;
  pc3= &c3;
  test_char_pc123(op, result);

}
#endif


#ifdef TEST_INT

void test_int_i123(char op, char *result){
  char r[20];

  switch(op){
  case '+':  i3 = i1+i2;  break;
  case '-':  i3 = i1-i2;  break;
  case '*':  i3 = i1*i2;  break;
  case '/':  i3 = i1/i2;  break;
  case '%':  i3 = i1%i2;  break;
  case '&':  i3 = i1&i2;  break;
  case '|':  i3 = i1|i2;  break;
  case '^':  i3 = i1^i2;  break;
  case '<':  i3 = i1<<(unsigned char)i2;  break;
  case '>':  i3 = i1>>(unsigned char)i2;  break;
  case '~':  i3 = ~i1;  break;
  case '!':  i3 = !i1;  break;
  }
  sprintf(r,"%04X",i3);
  printf("\nTest static INT(%c): %04X %c %04X = %04X (%s) %s\n",
                 op,i1,op,i2,i3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

}



void test_int_pi123(char op, char *result){
  char r[20];

  int  *api1, *api2, *api3;

  switch(op){
  case '+':  *pi3 = *pi1+ *pi2;  break;
  case '-':  *pi3 = *pi1- *pi2;  break;
  case '*':  *pi3 = *pi1* *pi2;  break;
  case '/':  *pi3 = *pi1/ *pi2;  break;
  case '%':  *pi3 = *pi1% *pi2;  break;
  case '&':  *pi3 = *pi1& *pi2;  break;
  case '|':  *pi3 = *pi1| *pi2;  break;
  case '^':  *pi3 = *pi1^ *pi2;  break;
  case '<':  *pi3 = *pi1<<(unsigned char)(*pi2);  break;
  case '>':  *pi3 = *pi1>>(unsigned char)(*pi2);  break;
  case '~':  *pi3 = ~*pi1;  break;
  case '!':  *pi3 = !*pi1;  break;
  }
  sprintf(r,"%04X",*pi3);
  printf("\nTest static * INT(%c): %04X %c %04X = %04X (%s) %s\n",
                 op,*pi1,op,*pi2,*pi3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

api1=pi1;
api2=pi2;
api3=pi3;

switch(op){
case '+':  *api3 = *api1+ *api2;  break;
case '-':  *api3 = *api1- *api2;  break;
case '*':  *api3 = *api1* *api2;  break;
case '/':  *api3 = *api1/ *api2;  break;
case '%':  *api3 = *api1% *api2;  break;
case '&':  *api3 = *api1& *api2;  break;
case '|':  *api3 = *api1| *api2;  break;
case '^':  *api3 = *api1^ *api2;  break;
case '<':  *api3 = *api1<<(unsigned char)(*api2);  break;
case '>':  *api3 = *api1>>(unsigned char)(*api2);  break;
case '~':  *api3 = ~*api1;  break;
case '!':  *api3 = !*api1;  break;
}
sprintf(r,"%04X",*api3);
printf("\nTest auto * INT(%c): %04X %c %04X = %04X (%s) %s\n",
               op,*api1,op,*api2,*api3,result,
               strcmp(result, r) ? "ERROR!" : "OK"
       );
DebBlink(1);
if( strcmp(result, r)) monitor();

}


void test_int(char op, int pai1, int pai2, char *result){
  char r[20];
  int pai3;

  switch(op){
  case '+':  pai3 = pai1+ pai2;  break;
  case '-':  pai3 = pai1- pai2;  break;
  case '*':  pai3 = pai1* pai2;  break;
  case '/':  pai3 = pai1/ pai2;  break;
  case '%':  pai3 = pai1% pai2;  break;
  case '&':  pai3 = pai1& pai2;  break;
  case '|':  pai3 = pai1| pai2;  break;
  case '^':  pai3 = pai1^ pai2;  break;
  case '<':  pai3 = pai1<<(unsigned char)pai2;  break;
  case '>':  pai3 = pai1>>(unsigned char)pai2;  break;
  case '~':  pai3 = ~pai1;  break;
  case '!':  pai3 = !pai1;  break;
  }
  sprintf(r,"%04X",pai3 );
  printf("\nTest auto INT(%c): %04X %c %04X = %04X (%s) %s\n",
                 op,pai1,op,pai2,pai3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

  i1= pai1;
  i2= pai2;
  test_int_i123(op, result);

  i1= pai1;
  i2= pai2;
  pi1= &i1;
  pi2= &i2;
  pi3= &i3;
  test_int_pi123(op, result);

}
#endif

#ifdef TEST_UINT

void test_uint_ui123(char op, char *result){
  char r[20];

  switch(op){
  case '+':  ui3 = ui1+ui2;  break;
  case '-':  ui3 = ui1-ui2;  break;
  case '*':  ui3 = ui1*ui2;  break;
  case '/':  ui3 = ui1/ui2;  break;
  case '%':  ui3 = ui1%ui2;  break;
  case '&':  ui3 = ui1&ui2;  break;
  case '|':  ui3 = ui1|ui2;  break;
  case '^':  ui3 = ui1^ui2;  break;
  case '<':  ui3 = ui1<<(unsigned char)ui2;  break;
  case '>':  ui3 = ui1>>(unsigned char)ui2;  break;
  case '~':  ui3 = ~ui1;  break;
  case '!':  ui3 = !ui1;  break;
  }
  sprintf(r,"%04X",ui3);
  printf("\nTest static UINT(%c): %04X %c %04X = %04X (%s) %s\n",
                 op,ui1,op,ui2,ui3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

}



void test_uint_pui123(char op, char *result){
  char r[20];

  unsigned int  *apui1, *apui2, *apui3;

  switch(op){
  case '+':  *pui3 = *pui1+ *pui2;  break;
  case '-':  *pui3 = *pui1- *pui2;  break;
  case '*':  *pui3 = *pui1* *pui2;  break;
  case '/':  *pui3 = *pui1/ *pui2;  break;
  case '%':  *pui3 = *pui1% *pui2;  break;
  case '&':  *pui3 = *pui1& *pui2;  break;
  case '|':  *pui3 = *pui1| *pui2;  break;
  case '^':  *pui3 = *pui1^ *pui2;  break;
  case '<':  *pui3 = *pui1<<(unsigned char)(*pui2);  break;
  case '>':  *pui3 = *pui1>>(unsigned char)(*pui2);  break;
  case '~':  *pui3 = ~*pui1;  break;
  case '!':  *pui3 = !*pui1;  break;
  }
  sprintf(r,"%04X",*pui3);
  printf("\nTest static * UINT(%c): %04X %c %04X = %04X (%s) %s\n",
                 op,*pui1,op,*pui2,*pui3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

apui1=pui1;
apui2=pui2;
apui3=pui3;

switch(op){
case '+':  *apui3 = *apui1+ *apui2;  break;
case '-':  *apui3 = *apui1- *apui2;  break;
case '*':  *apui3 = *apui1* *apui2;  break;
case '/':  *apui3 = *apui1/ *apui2;  break;
case '%':  *apui3 = *apui1% *apui2;  break;
case '&':  *apui3 = *apui1& *apui2;  break;
case '|':  *apui3 = *apui1| *apui2;  break;
case '^':  *apui3 = *apui1^ *apui2;  break;
case '<':  *apui3 = *apui1<<(unsigned char)(*apui2);  break;
case '>':  *apui3 = *apui1>>(unsigned char)(*apui2);  break;
case '~':  *apui3 = ~*apui1;  break;
case '!':  *apui3 = !*apui1;  break;
}
sprintf(r,"%04X",*apui3);
printf("\nTest auto * UINT(%c): %04X %c %04X = %04X (%s) %s\n",
               op,*apui1,op,*apui2,*apui3,result,
               strcmp(result, r) ? "ERROR!" : "OK"
       );
DebBlink(1);
if( strcmp(result, r)) monitor();

}


void test_uint(char op, unsigned int paui1, unsigned int paui2, char *result){
  char r[20];
  unsigned int paui3;

  switch(op){
  case '+':  paui3 = paui1+ paui2;  break;
  case '-':  paui3 = paui1- paui2;  break;
  case '*':  paui3 = paui1* paui2;  break;
  case '/':  paui3 = paui1/ paui2;  break;
  case '%':  paui3 = paui1% paui2;  break;
  case '&':  paui3 = paui1& paui2;  break;
  case '|':  paui3 = paui1| paui2;  break;
  case '^':  paui3 = paui1^ paui2;  break;
  case '<':  paui3 = paui1<<(unsigned char)paui2;  break;
  case '>':  paui3 = paui1>>(unsigned char)paui2;  break;
  case '~':  paui3 = ~paui1;  break;
  case '!':  paui3 = !paui1;  break;
  }
  sprintf(r,"%04X",paui3 );
  printf("\nTest auto UINT(%c): %04X %c %04X = %04X (%s) %s\n",
                 op,paui1,op,paui2,paui3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

  ui1= paui1;
  ui2= paui2;
  test_uint_ui123(op, result);

  ui1= paui1;
  ui2= paui2;
  pui1= &ui1;
  pui2= &ui2;
  pui3= &ui3;
  test_uint_pui123(op, result);

}
#endif



#ifdef TEST_LONG

void test_long_l123(char op, char *result){
  char r[20];

  switch(op){
  case '+':  l3 = l1+l2;  break;
  case '-':  l3 = l1-l2;  break;
  case '*':  l3 = l1*l2;  break;
  case '/':  l3 = l1/l2;  break;
  case '%':  l3 = l1%l2;  break;
  case '&':  l3 = l1&l2;  break;
  case '|':  l3 = l1|l2;  break;
  case '^':  l3 = l1^l2;  break;
  case '<':  l3 = l1<<(unsigned char)l2;  break;
  case '>':  l3 = l1>>(unsigned char)l2;  break;
  case '~':  l3 = ~l1;  break;
  case '!':  l3 = !l1;  break;
  }
  sprintf(r,"%08lX",l3);
  printf("\nTest static LONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
                 op,l1,op,l2,l3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

}



void test_long_pl123(char op, char *result){
  char r[20];

  long *apl1, *apl2, *apl3;

  switch(op){
  case '+':  *pl3 = *pl1+ *pl2;  break;
  case '-':  *pl3 = *pl1- *pl2;  break;
  case '*':  *pl3 = *pl1* *pl2;  break;
  case '/':  *pl3 = *pl1/ *pl2;  break;
  case '%':  *pl3 = *pl1% *pl2;  break;
  case '&':  *pl3 = *pl1& *pl2;  break;
  case '|':  *pl3 = *pl1| *pl2;  break;
  case '^':  *pl3 = *pl1^ *pl2;  break;
  case '<':  *pl3 = *pl1<<(unsigned char)(*pl2);  break;
  case '>':  *pl3 = *pl1>>(unsigned char)(*pl2);  break;
  case '~':  *pl3 = ~*pl1;  break;
  case '!':  *pl3 = !*pl1;  break;
  }
  sprintf(r,"%08lX",*pl3);
  printf("\nTest static * LONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
                 op,*pl1,op,*pl2,*pl3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

apl1=pl1;
apl2=pl2;
apl3=pl3;

switch(op){
case '+':  *apl3 = *apl1+ *apl2;  break;
case '-':  *apl3 = *apl1- *apl2;  break;
case '*':  *apl3 = *apl1* *apl2;  break;
case '/':  *apl3 = *apl1/ *apl2;  break;
case '%':  *apl3 = *apl1% *apl2;  break;
case '&':  *apl3 = *apl1& *apl2;  break;
case '|':  *apl3 = *apl1| *apl2;  break;
case '^':  *apl3 = *apl1^ *apl2;  break;
case '<':  *apl3 = *apl1<<(unsigned char)(*apl2);  break;
case '>':  *apl3 = *apl1>>(unsigned char)(*apl2);  break;
case '~':  *apl3 = ~*apl1;  break;
case '!':  *apl3 = !*apl1;  break;
}
sprintf(r,"%08lX",*apl3);
printf("\nTest auto * LONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
               op,*apl1,op,*apl2,*apl3,result,
               strcmp(result, r) ? "ERROR!" : "OK"
       );
DebBlink(1);
if( strcmp(result, r)) monitor();

}


void test_long(char op, long pal1, long pal2, char *result){
  char r[20];
  long pal3;

  switch(op){
  case '+':  pal3 = pal1+ pal2;  break;
  case '-':  pal3 = pal1- pal2;  break;
  case '*':  pal3 = pal1* pal2;  break;
  case '/':  pal3 = pal1/ pal2;  break;
  case '%':  pal3 = pal1% pal2;  break;
  case '&':  pal3 = pal1& pal2;  break;
  case '|':  pal3 = pal1| pal2;  break;
  case '^':  pal3 = pal1^ pal2;  break;
  case '<':  pal3 = pal1<<(unsigned char)pal2;  break;
  case '>':  pal3 = pal1>>(unsigned char)pal2;  break;
  case '~':  pal3 = ~pal1;  break;
  case '!':  pal3 = !pal1;  break;
  }
  sprintf(r,"%08lX",pal3 );
  printf("\nTest auto LONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
                 op,pal1,op,pal2,pal3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

  l1= pal1;
  l2= pal2;
  test_long_l123(op, result);

  l1= pal1;
  l2= pal2;
  pl1= &l1;
  pl2= &l2;
  pl3= &l3;
  test_long_pl123(op, result);


}
#endif



#ifdef TEST_ULONG

void test_ulong_ul123(char op, char *result){
  char r[20];

  switch(op){
  case '+':  ul3 = ul1+ul2;  break;
  case '-':  ul3 = ul1-ul2;  break;
  case '*':  ul3 = ul1*ul2;  break;
  case '/':  ul3 = ul1/ul2;  break;
  case '%':  ul3 = ul1%ul2;  break;
  case '&':  ul3 = ul1&ul2;  break;
  case '|':  ul3 = ul1|ul2;  break;
  case '^':  ul3 = ul1^ul2;  break;
  case '<':  ul3 = ul1<<(unsigned char)ul2;  break;
  case '>':  ul3 = ul1>>(unsigned char)ul2;  break;
  case '~':  ul3 = ~ul1;  break;
  case '!':  ul3 = !ul1;  break;
  }
  sprintf(r,"%08lX",ul3);
  printf("\nTest static ULONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
                 op,ul1,op,ul2,ul3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

}



void test_ulong_pul123(char op, char *result){
  char r[20];

  unsigned long *apul1, *apul2, *apul3;

  switch(op){
  case '+':  *pul3 = *pul1+ *pul2;  break;
  case '-':  *pul3 = *pul1- *pul2;  break;
  case '*':  *pul3 = *pul1* *pul2;  break;
  case '/':  *pul3 = *pul1/ *pul2;  break;
  case '%':  *pul3 = *pul1% *pul2;  break;
  case '&':  *pul3 = *pul1& *pul2;  break;
  case '|':  *pul3 = *pul1| *pul2;  break;
  case '^':  *pul3 = *pul1^ *pul2;  break;
  case '<':  *pul3 = *pul1<<(unsigned char)(*pul2);  break;
  case '>':  *pul3 = *pul1>>(unsigned char)(*pul2);  break;
  case '~':  *pul3 = ~*pul1;  break;
  case '!':  *pul3 = !*pul1;  break;
  }
  sprintf(r,"%08lX",*pul3);
  printf("\nTest static * ULONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
                 op,*pul1,op,*pul2,*pul3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

apul1=pul1;
apul2=pul2;
apul3=pul3;

switch(op){
case '+':  *apul3 = *apul1+ *apul2;  break;
case '-':  *apul3 = *apul1- *apul2;  break;
case '*':  *apul3 = *apul1* *apul2;  break;
case '/':  *apul3 = *apul1/ *apul2;  break;
case '%':  *apul3 = *apul1% *apul2;  break;
case '&':  *apul3 = *apul1& *apul2;  break;
case '|':  *apul3 = *apul1| *apul2;  break;
case '^':  *apul3 = *apul1^ *apul2;  break;
case '<':  *apul3 = *apul1<<(unsigned char)(*apul2);  break;
case '>':  *apul3 = *apul1>>(unsigned char)(*apul2);  break;
case '~':  *apul3 = ~*apul1;  break;
case '!':  *apul3 = !*apul1;  break;
}
sprintf(r,"%08lX",*apul3);
printf("\nTest auto * ULONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
               op,*apul1,op,*apul2,*apul3,result,
               strcmp(result, r) ? "ERROR!" : "OK"
       );
DebBlink(1);
if( strcmp(result, r)) monitor();

}


void test_ulong(char op, unsigned long paul1, unsigned long paul2, char *result){
  char r[20];
  unsigned long paul3;

  switch(op){
  case '+':  paul3 = paul1+ paul2;  break;
  case '-':  paul3 = paul1- paul2;  break;
  case '*':  paul3 = paul1* paul2;  break;
  case '/':  paul3 = paul1/ paul2;  break;
  case '%':  paul3 = paul1% paul2;  break;
  case '&':  paul3 = paul1& paul2;  break;
  case '|':  paul3 = paul1| paul2;  break;
  case '^':  paul3 = paul1^ paul2;  break;
  case '<':  paul3 = paul1<<(unsigned char)paul2;  break;
  case '>':  paul3 = paul1>>(unsigned char)paul2;  break;
  case '~':  paul3 = ~paul1;  break;
  case '!':  paul3 = !paul1;  break;
  }
  sprintf(r,"%08lX",paul3 );
  printf("\nTest auto ULONG(%c): %08lX %c %08lX = %08lX (%s) %s\n",
                 op,paul1,op,paul2,paul3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);
  if( strcmp(result, r)) monitor();

  ul1= paul1;
  ul2= paul2;
  test_ulong_ul123(op, result);

  ul1= paul1;
  ul2= paul2;
  pul1= &ul1;
  pul2= &ul2;
  pul3= &ul3;
  test_ulong_pul123(op, result);


}
#endif



#ifdef TEST_FLOAT
void test_float(char oper, float op1, float op2, char *result){
  char r[20];

  f1 = op1;
  f2 = op2;

  switch(oper){
  case '+':  f3 = f1+f2;  break;
  case '-':  f3 = f1-f2;  break;
  case '*':  f3 = f1*f2;  break;
  case '/':  f3 = f1/f2;  break;
  case 'S':  f3 = sinf(f1/f2);  break;
  case 'C':  f3 = cosf(f1/f2);  break;
  //case '2':  f3 = atan2f(f1,f2);  break;
  }

  sprintf(r,"%f",f3);

  printf("\nTest FLOAT(%c): %f %c %f = %f (%s) %s\n",
                 oper,f1,oper,f2,f3,result,
                 strcmp(result, r) ? "ERROR!" : "OK"
         );
  DebBlink(1);

  if( strcmp(result, r)) monitor();
}
#endif


void main(void)
{
  DebBlink(1);

  for(;;){

    monitor();


#ifdef TEST_CHAR
   test_char('+',0xB1,0x2C,"DD");
   test_char('+',0xB1,0xf0,"A1");
   test_char('-',0xB1,0x2C,"85");
   test_char('*',0xB1,0x2C,"6C");
   test_char('/',0xB1,0x2C,"FF");
   test_char('%',0xB1,0x2C,"DD");
   test_char('&',0xB1,0x2C,"20");
   test_char('|',0xB1,0x2C,"BD");
   test_char('^',0xB1,0x2C,"9D");
   test_char('<',0xB1,3,    "88");
   test_char('>',0xB1,3,    "F6");
   test_char('~',0xB1,0,    "4E");
   test_char('!',0xB1,0,    "00");
   test_char('!',0x00,0,    "01");
#endif

#ifdef TEST_UCHAR
   test_uchar('+',0xB1,0x2C,"DD");
   test_uchar('+',0xB1,0xf0,"A1");
   test_uchar('-',0xB1,0x2C,"85");
   test_uchar('*',0xB1,0x2C,"6C");
   test_uchar('/',0xB1,0x2C,"04");
   test_uchar('%',0xB1,0x2C,"01");
   test_uchar('&',0xB1,0x2C,"20");
   test_uchar('|',0xB1,0x2C,"BD");
   test_uchar('^',0xB1,0x2C,"9D");
   test_uchar('<',0xB1,3,    "88");
   test_uchar('>',0xB1,3,    "16");
   test_uchar('~',0xB1,0,    "4E");
   test_uchar('!',0xB1,0,    "00");
   test_uchar('!',0x00,0,    "01");
#endif


#ifdef TEST_INT
    test_int('+',0xB1A0,0x2C3D,"DDDD");
    test_int('+',0xB1A0,0xf0f0,"A290");
    test_int('-',0xB1A0,0x2C3D,"8563");
    test_int('*',0xB1A0,0x2C3D,"D320");
    test_int('/',0xB1A0,0x2C3D,"FFFF");
    test_int('%',0xB1A0,0x2C3D,"DDDD");
    test_int('&',0xB1A0,0x2C3D,"2020");
    test_int('|',0xB1A0,0x2C3D,"BDBD");
    test_int('^',0xB1A0,0x2C3D,"9D9D");
    test_int('<',0xB1A0,3,      "8D00");
    test_int('>',0xB1A0,3,      "F634");
    test_int('~',0xB1A0,0,      "4E5F");
    test_int('!',0xB1A0,0,      "0000");
    test_int('!',0x0000,0,      "0001");
#endif


#ifdef TEST_UINT
    test_uint('+',0xB1A0,0x2C3D,"DDDD");
    test_uint('+',0xB1A0,0xf0f0,"A290");
    test_uint('-',0xB1A0,0x2C3D,"8563");
    test_uint('*',0xB1A0,0x2C3D,"D320"); // should be CEA0, but no such a mul
    test_uint('/',0xB1A0,0x2C3D,"0004");
    test_uint('%',0xB1A0,0x2C3D,"00AC");
    test_uint('&',0xB1A0,0x2C3D,"2020");
    test_uint('|',0xB1A0,0x2C3D,"BDBD");
    test_uint('^',0xB1A0,0x2C3D,"9D9D");
    test_uint('<',0xB1A0,3,      "8D00");
    test_uint('>',0xB1A0,3,      "1634");
    test_uint('~',0xB1A0,0,      "4E5F");
    test_uint('!',0xB1A0,0,      "0000");
    test_uint('!',0x0000,0,      "0001");
#endif

#ifdef TEST_LONG
    test_long('+',0xD3C2B1A0,0x0A1B2C3D,"DDDDDDDD");
    test_long('+',0xD3C2B1A0,0x00f0f0f0,"D4B3A290");
    test_long('-',0xD3C2B1A0,0x0A1B2C3D,"C9A78563");
    test_long('*',0xD3C2B1A0,0x0A1B2C3D,"E7CBD320");
    test_long('/',0xD3C2B1A0,0x0A1B2C3D,"FFFFFFFC");
    test_long('%',0xD3C2B1A0,0x0A1B2C3D,"FC2F6294");//+3D09D6C
    test_long('&',0xD3C2B1A0,0x0A1B2C3D,"02022020");
    test_long('|',0xD3C2B1A0,0x0A1B2C3D,"DBDBBDBD");
    test_long('^',0xD3C2B1A0,0x0A1B2C3D,"D9D99D9D");
    test_long('<',0xD3C2B1A0,3,           "9E158D00");
    test_long('>',0xD3C2B1A0,3,           "FA785634");
    test_long('~',0xD3C2B1A0,0,           "2C3D4E5F");
    test_long('!',0xD3C2B1A0,0,           "00000000");
    test_long('!',0x00000000,0,           "00000001");
#endif

#ifdef TEST_ULONG
    test_ulong('+',0xD3C2B1A0,0x0A1B2C3D,"DDDDDDDD");
    test_ulong('+',0xD3C2B1A0,0x00f0f0f0,"D4B3A290");
    test_ulong('-',0xD3C2B1A0,0x0A1B2C3D,"C9A78563");
    test_ulong('*',0xD3C2B1A0,0x0A1B2C3D,"E7CBD320");
    test_ulong('/',0xD3C2B1A0,0x0A1B2C3D,"00000014");
    test_ulong('%',0xD3C2B1A0,0x0A1B2C3D,"09A33CDC");
    test_ulong('&',0xD3C2B1A0,0x0A1B2C3D,"02022020");
    test_ulong('|',0xD3C2B1A0,0x0A1B2C3D,"DBDBBDBD");
    test_ulong('^',0xD3C2B1A0,0x0A1B2C3D,"D9D99D9D");
    test_ulong('<',0xD3C2B1A0,3,           "9E158D00");
    test_ulong('>',0xD3C2B1A0,3,           "1A785634");
    test_ulong('~',0xD3C2B1A0,0,           "2C3D4E5F");
    test_ulong('!',0xD3C2B1A0,0,           "00000000");
    test_ulong('!',0x00000000,0,           "00000001");
#endif

#ifdef TEST_FLOAT
    test_float('+',3.14,11.11,"14.249999");
    test_float('-',3.14,11.11,"-7.970000");
    test_float('*',3.14,11.11,"34.885398");
    test_float('/',3.14,11.11,"0.282628");
    test_float('S',PI,4.0,"0.707106");
    test_float('C',PI,4.0,"0.707106");
    test_float('2',-PI,-PI/2,"0.707106");
#endif

#ifdef TEST_BITS
  {
    reg |= 8;
    reg |= 0x8;

    creg |= 0x1;
    if( creg != 0x44) {
      reg &= 0xFFFE;
    }
  }
#endif

#ifdef TEST_SWITCH

  for( c1=0; c1<=10; c1++ ){

    switch(c1){
    case 0: printf("sw:%d",(int)c1); break;
    case 1: printf("sw:%d",(int)c1); break;
    case 2: printf("sw:%d",(int)c1); break;
    case 3: printf("sw:%d",(int)c1); break;
    case 4: printf("sw:%d",(int)c1); break;
    case 5: printf("sw:%d",(int)c1); break;
    case 6: printf("sw:%d",(int)c1); break;
    case 7: printf("sw:%d",(int)c1); break;
    case 8: printf("sw:%d",(int)c1); break;
    case 9: printf("sw:%d",(int)c1); break;
    default: printf("def:%d",(int)c1); break;
    }
  }   


#endif
  printf("\nThe End\n");
  DebBlink(1);
 
  } // for
}


