/*
** libgcc support for software floating point.
** Copyright (C) 1991 by Pipeline Associates, Inc.  All rights reserved.
** Permission is granted to do *anything* you want with this file,
** commercial or otherwise, provided this message remains intact.  So there!
** I would appreciate receiving any updates/patches/changes that anyone
** makes, and am willing to be the repository for said changes (am I
** making a big mistake?).
**
** Pat Wood
** Pipeline Associates, Inc.
** pipeline!phw@motown.com or
** sun!pipeline!phw or
** uunet!motown!pipeline!phw
*/

#include <float.h>

union float_long
  {
    float f;
    unsigned long l;
  };


/* add two floats */
float __fsadd (float a1, float a2)
{
  long mant1, mant2;
  union float_long fl1, fl2;
  int exp1, exp2;
  unsigned char sign = 0;

  fl1.f = a1;
  fl2.f = a2;

  /* check for zero args */

  if (!fl1.l)
    return (fl2.f);
  if (!fl2.l)
    return (fl1.f);

  exp1 = EXP (fl1.l);
  exp2 = EXP (fl2.l);

  if (exp1 > exp2 + 25)
    return (fl1.f);
  if (exp2 > exp1 + 25)
    return (fl2.f);

  mant1 = MANT (fl1.l);
  mant2 = MANT (fl2.l);

  if (__BYTE(fl1.l,0) &__SIGNBIT)
    mant1 = -mant1;
  if (__BYTE(fl2.l,0) &__SIGNBIT )
    mant2 = -mant2;

  if (exp1 > exp2){
      mant2 >>= exp1 - exp2;
  } else {
      mant1 >>= exp2 - exp1;
      exp1 = exp2;
  }
  mant1 += mant2;

  if ( __BYTE(mant1,0) < 0){
      mant1 = -mant1;
      sign = __SIGNBIT;
  }
  else if (!mant1)
    return (0);

  /* normalize */
  while (mant1<HIDDEN) {
    mant1 <<= 1;
    exp1--;
  }

  /* round off */
  while ( __BYTE(mant1,0) ) {
    if( __BYTE(mant1,3) & 1 )
      mant1 += 2;
    mant1 >>= 1 ;
    exp1++;
  }

  return PACK (sign, exp1, mant1);
}
