/*-------------------------------------------------------------------------
  vprintf.c - formatted output conversion

             Written By - Martijn van Balen aed@iae.nl (1999)
	     Added %f By - johan.knol@iduna.nl (2000)

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
-------------------------------------------------------------------------*/

#define USE_FLOATS 1

/* this module uses some global variables instead function parameters, so: */
#ifdef SDCC_STACK_AUTO
#warning "this module cannot yet be use as a reentrant one"
#endif

#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <float.h>



#define PTR value.p

#define NULL_STRING "<NULL>"
#define NULL_STRING_LENGTH 6

/****************************************************************************/

//typedef char * ptr_t;
#define ptr_t char *

#ifdef toupper
#undef toupper
#endif

//#define toupper(c) ((c)&=~0x20)
#define toupper(c) ((c)&=0xDF)

typedef union
{
  unsigned char  byte[5];
  struct { char byte; long  l; } l;
  //struct { char byte; unsigned long  ul; } ul;
  float f;
  char           *p;
} value_t;


static code char memory_id[] = "IXCP-";

static ptr_t output_ptr;
static bit   output_to_string;
static bit   lower_case;
/* this one NEEDS to be in data */
static data value_t value;

static unsigned char radix;

// this makes the whole dammed thing nonreentrent
static int charsOutputted;

/****************************************************************************/

static void output_char( char c ) reentrant
{
  if (output_to_string)
  {
    *output_ptr++ = c;
  }
  else
  {
    putchar( c );
  }
  charsOutputted++;
}

/*--------------------------------------------------------------------------*/

static void output_digit( unsigned char n ) reentrant
{
  output_char( n <= 9 ? '0'+n :
	       (lower_case ? n+(char)('a'-10) : n+(char)('A'-10)) );
}

/*--------------------------------------------------------------------------*/

static void output_2digits( unsigned char b ) reentrant
{
  output_digit( b>>4 );
  output_digit( b&0x0F );
}

/*--------------------------------------------------------------------------*/

static void calculate_digit( void )
{
  unsigned char i;

  value.byte[0]=0;
  for( i = 32; i != 0; i-- ) {

    _asm
      sta  0xffff
      ldhx #_value
      asl  4,x
      rol  3,x
      rol  2,x
      rol  1,x
      rol  0,x
    _endasm;

    if (radix <= value.byte[0] ){
       value.byte[0] -= radix;
       value.byte[4]++;
    }
  }
}

#if USE_FLOATS

/* This is a very inefficient but direct approach, since we have no math
   library yet (e.g. log()).
   It does most of the modifiers, but has some restrictions. E.g. the
   abs(float) shouldn't be bigger than an unsigned long (that's
   about 4294967295), but still makes it usefull for most real-life
   applications.
*/


#define DEFAULT_FLOAT_PRECISION 6

static void output_float (float f, unsigned char reqWidth,
			  signed char reqDecimals,
			  bit left, bit zero, bit sign, bit space)
{
  char negative=0;
  unsigned long integerPart;
  float decimalPart;
  static char fpBuffer[128];
  char fpBI=0, fpBD;
  unsigned char minWidth, i;

  // save the sign
  if (f<0) {
    negative=1;
    f=-f;
  }

  if (f>0x00ffffff) {
    // this part is from Frank van der Hulst
    signed char exp;

    for (exp = 0; f >= 10.0; exp++) f /=10.0;
    for (       ; f < 1.0;   exp--) f *=10.0;

    if (negative) {
      putchar ('-');
    } else {
      if (sign) {
	putchar ('+');
      }
    }
    output_float(f, 0, reqDecimals, 0, 0, 0, 0);
    putchar ('e');
    if (exp<0) {
      putchar ('-');
      exp = -exp;
    }
    putchar ('0'+exp/10);
    putchar ('0'+exp%10);
    return;
  }

  // split the float
  integerPart=f;
  decimalPart=f-integerPart;

  // fill the buffer with the integerPart (in reversed order!)
  while (integerPart) {
    fpBuffer[fpBI++]='0' +(unsigned char)( integerPart%10);
    integerPart /= 10;
  }
  if (!fpBI) {
    // we need at least a 0
    fpBuffer[fpBI++]='0';
  }

  // display some decimals as default
  if (reqDecimals==-1)
    reqDecimals=DEFAULT_FLOAT_PRECISION;

  // fill buffer with the decimalPart (in normal order)
  fpBD=fpBI;
  if (i=reqDecimals /* that's an assignment */) {
    do {
      decimalPart *= 10.0;
      // truncate the float
      integerPart=decimalPart;
      fpBuffer[fpBD++]='0' + integerPart;
      decimalPart-=integerPart;
    } while (--i);
  }

  minWidth=fpBI; // we need at least these
  minWidth+=reqDecimals?reqDecimals+1:0; // maybe these
  if (negative || sign || space)
    minWidth++; // and maybe even this :)

  if (!left && reqWidth>i) {
    if (zero) {
      if (negative) output_char('-');
      else if (sign) output_char('+');
      else if (space) output_char(' ');
      while (reqWidth-->minWidth)
	output_char ('0');
    } else {
      while (reqWidth-->minWidth)
	output_char (' ');
      if (negative) output_char('-');
      else if (sign) output_char('+');
      else if (space) output_char (' ');
    }
  } else {
    if (negative) output_char('-');
    else if (sign) output_char('+');
    else if (space) output_char(' ');
  }

  // output the integer part
  i=fpBI-1;
  do {
    output_char (fpBuffer[i]);
  } while (i--);

  // ouput the decimal part
  if (reqDecimals) {
    output_char ('.');
    i=fpBI;
    while (reqDecimals--)
      output_char (fpBuffer[i++]);
  }

  if (left && reqWidth>minWidth) {
    while (reqWidth-->minWidth)
      output_char(' ');
  }
}
#endif

/*--------------------------------------------------------------------------*/

int vsprintf (const char *buf, const char *format, va_list ap)
{
  unsigned char  left_justify;
  unsigned char  zero_padding;
  unsigned char  prefix_sign;
  unsigned char  prefix_space;
  unsigned char  signed_argument;
  unsigned char  char_argument;
  unsigned char  long_argument;
  unsigned char  float_argument;

  unsigned char  width;
  signed char decimals;
  unsigned char  length;
  char           c;

  #define DigBufferLen 20
  char DigBuffer[DigBufferLen];
  char *DigPtr;

  // reset output chars
  charsOutputted=0;

  output_ptr = buf;
  if ( !buf )
  {
    output_to_string = 0;
  }
  else
  {
    output_to_string = 1;
  }

  while( c=*format++ )
  {
    _asm
      sta 0xffff
    _endasm;

    if ( c=='%' ){
      left_justify    = 0;
      zero_padding    = 0;
      prefix_sign     = 0;
      prefix_space    = 0;
      signed_argument = 0;
      radix           = 0;
      char_argument   = 0;
      long_argument   = 0;
      float_argument  = 0;
      width           = 0;
      decimals        = -1;

get_conversion_spec:

      c = *format++;

      if (c=='%') {
	output_char(c);
	continue;
      }

      if (isdigit(c)) {
	if (decimals==-1) {
	  width = 10*width + (c - '0');
	  if (width == 0) {
	    /* first character of width is a zero */
	    zero_padding = 1;
	  }
	} else {
	  decimals = 10*decimals + (c-'0');
	}
	goto get_conversion_spec;
      }

      if (c=='.') {
	if (decimals=-1) decimals=0;
	else
	  ; // duplicate, ignore
	goto get_conversion_spec;
      }

      lower_case = islower(c);
      if (lower_case){
	c = toupper(c);
      }

      switch( c ){
      case '-':
	left_justify = 1;
	goto get_conversion_spec;
      case '+':
	prefix_sign = 1;
	goto get_conversion_spec;
      case ' ':
	prefix_space = 1;
	goto get_conversion_spec;
      case 'B':
	char_argument = 1;
	goto get_conversion_spec;
      case 'L':
	long_argument = 1;
	goto get_conversion_spec;

      case 'C':
       	output_char( va_arg(ap,int) );
	break;

      case 'S':
	PTR = va_arg(ap,ptr_t);

	if (PTR==0) {
	  PTR=NULL_STRING;
	  length=NULL_STRING_LENGTH;
	} else {
	  length = strlen(PTR);
	}

	if ( ( !left_justify ) && (length < width) ){
	  width -= length;
	  while( width-- != 0 ){
	    output_char( ' ' );
	  }
	}

        while ( *PTR )
	  output_char( *PTR++ );

	if ( left_justify && (length < width)){
	  width -= length;
	  while( width-- != 0 ){
	    output_char( ' ' );
	  }
	}
	break;

      case 'P':
	PTR = va_arg(ap,ptr_t);
        output_char('P' );
        output_char(':');
        output_char('0');
        output_char('x');
        output_2digits( value.byte[4] );
        output_2digits( value.byte[3] );
	break;

      case 'D':
      case 'I':
	signed_argument = 1;
	radix = 10;
	break;

      case 'O':
	radix = 8;
	break;

      case 'U':
	radix = 10;
	break;

      case 'X':
	radix = 16;
	break;

      case 'F':
	float_argument=1;
	break;

      default:
	// nothing special, just output the character
	output_char( c );
	break;
      }

      if (float_argument) {
	value.f=va_arg(ap,float);
#if !USE_FLOATS
	PTR="<NO FLOAT>";
	while (c=*PTR++)
	  output_char (c);
	// treat as long hex
	//radix=16;
	//long_argument=1;
	//zero_padding=1;
	//width=8;
#else
	// ignore b and l conversion spec for now
	output_float(value.f, width, decimals, left_justify, zero_padding,
		     prefix_sign, prefix_space);
#endif
      } else if (radix != 0){
      
	// Apperently we have to output an integral type
        // with radix "radix"

        // store value in byte[0] (LSB) ... byte[3] (MSB)
	if (char_argument){
	  value.l.l = va_arg(ap,char);
	  if (!signed_argument) {
                  value.byte[1] = 0x00;
                  value.byte[2] = 0x00;
                  value.byte[3] = 0x00;
	  }
	} else if (long_argument)	{
	  value.l.l = va_arg(ap,long);
	} else { // must be int

	  value.l.l = va_arg(ap,int);
	  if (!signed_argument) {
            value.byte[1] = 0x00;
            value.byte[2] = 0x00;
	  }
	}

        if ( signed_argument ){
          if (value.l.l < 0){
            value.l.l = -value.l.l;
          } else {
            signed_argument = 0;
          }
        }

	length=0;
        DigPtr = DigBuffer;

	do {
	  calculate_digit();

          *DigPtr++ = value.byte[0];

	  length++;
        } while( (value.byte[1] != 0) || (value.byte[2] != 0) ||
                 (value.byte[3] != 0) || (value.byte[4] != 0) );

	if(width == 0){
	  // default width. We set it to 1 to output
          // at least one character is case the value itself
          // is zero (i.e. length==0)
	  width=1;
	}

	/* prepend spaces if needed */
	if (!zero_padding && !left_justify){
	  while ( width > length+1 ){
	    output_char( ' ' );
	    width--;
	  }
	}

        if (signed_argument){ // this now means the original value was negative

          output_char( '-' );
	  // adjust width to compensate for this character
	  width--;

        } else if (length != 0)	{
	  // value > 0
	  if (prefix_sign){
	    output_char( '+' );
	    // adjust width to compensate for this character
	    width--;
	  } else if (prefix_space){
	    output_char( ' ' );
	    // adjust width to compensate for this character
            width--;
	  }
	}

	/* prepend zeroes/spaces if needed */
	if (!left_justify){
	  while ( width-- > length ){
	    output_char( zero_padding ? '0' : ' ' );
	  }
	} else {
	  /* spaces are appended after the digits */
	  if (width > length){
	    width -= length;
	  } else {
	    width = 0;
	  }
	}

	/* output the digits */
	while( length-- ){
          output_digit( *--DigPtr );
	}
	if (left_justify)
	  while (width-- > 0)
	    output_char(' ');
      }
    } else {
      // nothing special, just output the character
      output_char( c );
    }
  }

  // Copy \0 to the end of buf
  // Modified by JB 17/12/99
  if (output_to_string) {
    output_char(0);
    return charsOutputted-1;
  } else {
    return charsOutputted;
  }
}

/*--------------------------------------------------------------------------*/

int vprintf (const char *format, va_list ap)
{
  return vsprintf( 0, format, ap );
}
